<?php
/**
 * Initialises the app
 *
 * @package Fast Plugin
 *
 * @copyright 2017 Oxygenna.com
 * @license http://themeforest.net/licenses/standard
 * @version 1.5.0
 * @author Oxygenna
 */

/**
 * Class to setup the app
 */
class FastTicketApp
{
    /**
     * Stores the singleton instance of the class
     *
     * @var FastTicketApp
     */
    private static $instance;

    /**
     * Creates a new instance of the class
     */
    public function __construct()
    {
        // Start the clean slate so we have WordPress all to ourselves.
        require FAST_PLUGIN_DIR . 'classes/class-clean-slate.php';
        FastCleanSlate::init();

        add_action('wp_enqueue_scripts_clean', array(&$this, 'enqueue_scripts_and_styles'));

        add_action('wp_clean_slate_head', array(&$this, 'add_base_tag'));

        add_action('wp_clean_slate_head', array(&$this, 'add_css_styling'));

        add_action('wp_clean_slate_content', array(&$this, 'add_content'));
    }

    /**
     * Enqueues app styles and scripts
     */
    public function enqueue_scripts_and_styles()
    {
        wp_enqueue_style('fast-style-google-fonts', 'https://fonts.googleapis.com/icon?family=Material+Icons|Roboto:300,400,400i,700');

        wp_register_script('fast-script-app_inline', FAST_PLUGIN_URI . 'assets/app/inline.bundle.js', array(), '1.5.0', true);
        $current_user = wp_get_current_user();

        $options = array(
            'nonce'                       => wp_create_nonce('ticket-api-action'),
            'private_tickets'             => fast_get_option('private_tickets'),
            'show_site_url'               => fast_get_option('show_site_url') === 'on',
            'assets_folder_url'           => FAST_PLUGIN_URI . 'assets/',
            'envato_login'                => fast_get_option('envato_login') === 'on',
            'ticket_comments_order'       => fast_get_option('ticket_comments_order'),
            'envato_create_ticket_when_supported' => fast_get_option('envato_create_ticket_when_supported') === 'on',
            'envato_login_url'            => fast_generate_envato_oauth_login_url(),
            'recaptcha_site_key'          => fast_get_option('recaptcha_site_key'),
            'register_page_title'         => fast_get_option('register_page_title'),
            'register_page_content'       => wpautop(fast_get_option('register_page_content')),
            'register_types'              => fast_get_option('register_types'),
            'login_page_title'            => fast_get_option('login_page_title'),
            'login_page_login_types'      => fast_get_option('login_page_login_types'),
            'login_page_content'          => wpautop(fast_get_option('login_page_content')),
            'new_ticket_type'             => fast_get_option('new_ticket_type'),
            'new_ticket_page_title'       => fast_get_option('new_ticket_page_title'),
            'new_ticket_page_content'     => wpautop(fast_get_option('new_ticket_page_content')),
            'extra_fields'                => fast_get_option('extra_fields'),
            'max_upload_size'             => round(wp_max_upload_size() / 1000000),
            'site_language'               => get_locale(),
            'available_languages'         => fast_get_app_available_languages(),
            'agent_ticket_creation'       => fast_get_option('agent_ticket_creation') === 'on',
            'register_user_language'      => fast_get_option('register_user_language') === 'on',
            'favicon_color'               => fast_get_option('favicon_color'),
            'favicon_bg_color'            => fast_get_option('favicon_bg_color'),
            'admin_url'                   => admin_url(),
            'wp_lostpassword_url'         => wp_lostpassword_url(),
            'customer_reply_to_closed'    => fast_get_option('customer_reply_to_closed') === 'on',
        );

        if (fast_is_woocommerce_active()) {
            $options['wc_order_status'] = wc_get_order_statuses();
        }

        wp_localize_script('fast-script-app_inline', 'wpLocal', array(
            'root'    => rest_url(),
            'nonce'   => wp_create_nonce('wp_rest'),
            'options' => $options
        ));

        wp_enqueue_style('fast-style-app_styles', FAST_PLUGIN_URI . 'assets/app/themes/' . fast_get_option('app_color') . '.bundle.css', array(), '1.5.0');

        wp_enqueue_script('fast-script-app_inline');
        wp_enqueue_script('fast-script-app_polyfills', FAST_PLUGIN_URI . 'assets/app/polyfills.bundle.js', array(), '1.5.0', true);
        wp_enqueue_script('fast-script-app_scripts', FAST_PLUGIN_URI . 'assets/app/scripts.bundle.js', array(), '1.5.0', true);
        wp_enqueue_script('fast-script-app_vendor', FAST_PLUGIN_URI . 'assets/app/vendor.bundle.js', array(), '1.5.0', true);
        wp_enqueue_script('fast-script-app_main', FAST_PLUGIN_URI . 'assets/app/main.bundle.js', array(), '1.5.0', true);
    }

    /**
     * Adds the base tag to the head (needed by angular)
     */
    public function add_base_tag()
    {
        echo '<base href="' . esc_url(get_the_permalink()) . '">';
        if (has_site_icon()) {
            wp_site_icon();
        }
    }

    /**
     * Adds the base tag to the head (needed by angular)
     */
    public function add_css_styling()
    {
        echo '<style type="text/css">' . fast_custom_css() . '</style>';
        echo '<style type="text/css" id="app-extra-css">' . fast_get_option('extra_css') . '</style>';
    }

    /**
     * Adds the app root to the content of the page
     */
    public function add_content()
    {
        echo '<tick-app><div class="app-spinner"><div class="loader"></div></div></tick-app>';
    }

    /**
     * Instance accessor. If instance doesn't exist, we'll initialize the class.
     *
     * @return object FastTicketApp
     */
    public static function instance()
    {
        if (!isset(self::$instance)) {
            self::$instance = new FastTicketApp;
        }
        return self::$instance;
    }
}
