<?php
/**
 * Actions for the plugin
 *
 * @package Fast Plugin
 *
 * @copyright 2017 Oxygenna.com
 * @license http://themeforest.net/licenses/standard
 * @version 1.5.0
 * @author Oxygenna
 */

/*
 * register_activation_hook
 */

/**
 * Run when the plugin activates.
 */
function fast_install_plugin()
{
    add_option('fast-plugin-activated', 'fast-plugin');
}
register_activation_hook(FAST_PLUGIN_FILE, 'fast_install_plugin');

/*
 * plugins_loaded
 */

/**
 * Load plugin textdomain.
 */
function fast_load_textdomain()
{
    load_plugin_textdomain('fast-plugin', false, 'fast-plugin/languages');
}
add_action('plugins_loaded', 'fast_load_textdomain');

/*
 * init
 */

/**
 * Adds custom user roles.
 */
function fast_add_user_roles()
{
    if (is_admin() &&  'fast-plugin' === get_option('fast-plugin-activated')) {
        delete_option('fast-plugin-activated');

        // Add ticket capabilities to administrator role.
        $caps = apply_filters('fast_filter_ticket_capabilities', array());
        $admin = get_role('administrator');
        foreach ($caps as $cap => $val) {
            $admin->add_cap($cap);
        }

        fast_create_plugin_default_roles();

        $taxonomies = array(
            'fast_status' => array(
                esc_html__('Open', 'fast-plugin'),
                esc_html__('Closed', 'fast-plugin'),
            )
        );
        foreach ($taxonomies as $taxonomy => $terms) {
            foreach ($terms as $term) {
                if (! get_term_by('slug', sanitize_title($term), $taxonomy)) {
                    $result = wp_insert_term($term, $taxonomy);
                }
            }
        }
        // Save Permalinks.
        flush_rewrite_rules();
    }
}
add_action('init', 'fast_add_user_roles', 100);

/**
 * Registers a custom post types.
 */
function fast_register_custom_posts()
{
    register_post_type('fast_ticket', array(
        'labels' => array(
            'name'          => esc_html__('Tickets', 'fast-plugin'),
            'singular_name' => esc_html__('Ticket', 'fast-plugin')
        ),
        'supports'              => array('title', 'author', 'custom-fields', 'comments'),
        'public'                => true,
        'has_archive'           => false,
        'menu_icon'             => 'dashicons-tickets',
        'show_in_rest'          => true,
        'rest_base'             => 'tickets',
        'rest_controller_class' => 'FastRESTTicketController',
        'rewrite'               => false,
        'exclude_from_search'   => true,
        'publicly_queryable'    => true,
        'capabilities' => array(
            'edit_post'              => 'edit_ticket',
            'read_post'              => 'read_ticket',
            'delete_post'            => 'delete_ticket',

            'edit_posts'             => 'edit_tickets',
            'edit_others_posts'      => 'edit_others_tickets',
            'publish_posts'          => 'publish_tickets',
            'read_private_posts'     => 'read_private_tickets',

            'read'                   => 'read',
            'delete_posts'           => 'delete_ticket',
            'delete_private_posts'   => 'delete_ticket',
            'delete_published_posts' => 'delete_ticket',
            'delete_others_posts'    => 'delete_ticket',
            'edit_private_posts'     => 'read_private_tickets',
            'edit_published_posts'   => 'read_private_tickets',
            'create_posts'           => 'edit_tickets',
        )
    ));

    register_post_type('fast_reply', array(
        'labels' => array(
            'name'          => esc_html__('پاسخ های آماده', 'fast-plugin'),
            'singular_name' => esc_html__('ذخیره یک پاسخ آماده', 'fast-plugin')
        ),
        'supports'              => array('title', 'editor', 'author'),
        'public'                => false,
        'has_archive'           => false,
        'menu_icon'             => 'dashicons-book-alt',
        'show_in_rest'          => true,
        'show_ui'               => true,
        'rest_base'             => 'replies',
        'rest_controller_class' => 'FastRESTTicketController',
        'rewrite'               => false,
        'exclude_from_search'   => true,
        'publicly_queryable'    => false,
        'capabilities' => array(
            'edit_post'              => 'edit_reply',
            'read_post'              => 'read_reply',
            'delete_post'            => 'delete_reply',

            'edit_posts'             => 'edit_replies',
            'edit_others_posts'      => 'edit_others_replies',
            'publish_posts'          => 'publish_replies',

            'read'                   => 'read',
            'delete_posts'           => 'delete_reply',
            'delete_private_posts'   => 'delete_reply',
            'delete_published_posts' => 'delete_reply',
            'delete_others_posts'    => 'delete_reply',
            'edit_private_posts'     => 'read_private_replies',
            'edit_published_posts'   => 'read_private_replies',
            'create_posts'           => 'edit_replies',
        )
    ));

    register_post_type('fast_purchase', array(
        'supports'              => array('author'),
        'public'                => false,
        'has_archive'           => false,
        'show_in_rest'          => true,
        'show_ui'               => false,
        'rest_base'             => 'envato_purchases',
        'rest_controller_class' => 'FastRESTTicketController',
        'rewrite'               => false,
        'exclude_from_search'   => true,
        'publicly_queryable'    => false,
        'capabilities' => array(
            'edit_post'              => 'edit_purchase',
            'read_post'              => 'read_purchase',
            'delete_post'            => 'delete_purchase',

            'edit_posts'             => 'edit_purchases',
            'edit_others_posts'      => 'edit_others_purchases',
            'publish_posts'          => 'publish_purchases',

            'read'                   => 'read',
            'delete_posts'           => 'delete_purchases',
            'delete_private_posts'   => 'delete_private_purchases',
            'delete_published_posts' => 'delete_published_purchases',
            'delete_others_posts'    => 'delete_others_purchases',
            'edit_private_posts'     => 'edit_private_purchases',
            'edit_published_posts'   => 'edit_published_purchases',
            'create_posts'           => 'edit_purchases',
        )
    ));
}
add_action('init', 'fast_register_custom_posts');

/**
 * Registers a custom post types.
 */
function fast_register_custom_taxonomies()
{
    register_taxonomy('fast_status', array('fast_ticket'), array(
        'labels'                => array(
            'name'              => esc_html_x('Statuses', 'taxonomy general name', 'fast-plugin'),
            'singular_name'     => esc_html_x('Status', 'taxonomy singular name', 'fast-plugin'),
            'search_items'      => esc_html__('Search Statuses', 'fast-plugin'),
            'all_items'         => esc_html__('All Statuses', 'fast-plugin'),
            'parent_item'       => esc_html__('Parent Status', 'fast-plugin'),
            'parent_item_colon' => esc_html__('Parent Status:', 'fast-plugin'),
            'edit_item'         => esc_html__('Edit Status', 'fast-plugin'),
            'update_item'       => esc_html__('Update Status', 'fast-plugin'),
            'add_new_item'      => esc_html__('Add New Status', 'fast-plugin'),
            'new_item_name'     => esc_html__('New Status Name', 'fast-plugin'),
            'menu_name'         => esc_html__('Statuses', 'fast-plugin'),
        ),
        'hierarchical'          => false,
        'show_ui'               => true,
        'show_in_menu'          => true,
        'show_admin_column'     => true,
        'query_var'             => true,
        'rewrite'               => false,
        'show_in_rest'          => true,
        'publicly_queryable'    => false,
        'rest_base'             => 'ticket_status',
        'rest_controller_class' => 'WP_REST_Terms_Controller',
        'update_count_callback' => '_update_generic_term_count',
        'capabilities'          => array(
            'manage_terms' => 'manage_statuses',
            'edit_terms'   => 'manage_statuses',
            'delete_terms' => 'manage_statuses',
            'assign_terms' => 'assign_status'
        )
    ));

    register_taxonomy('fast_product', array('fast_ticket'), array(
        'labels'                => array(
            'name'              => esc_html_x('Products', 'taxonomy general name', 'fast-plugin'),
            'singular_name'     => esc_html_x('Product', 'taxonomy singular name', 'fast-plugin'),
            'search_items'      => esc_html__('Search Products', 'fast-plugin'),
            'all_items'         => esc_html__('All Products', 'fast-plugin'),
            'parent_item'       => esc_html__('Parent Product', 'fast-plugin'),
            'parent_item_colon' => esc_html__('Parent Product:', 'fast-plugin'),
            'edit_item'         => esc_html__('Edit Product', 'fast-plugin'),
            'update_item'       => esc_html__('Update Product', 'fast-plugin'),
            'add_new_item'      => esc_html__('Add New Product', 'fast-plugin'),
            'new_item_name'     => esc_html__('New Product Name', 'fast-plugin'),
            'menu_name'         => esc_html__('Products', 'fast-plugin'),
        ),
        'hierarchical'          => false,
        'show_ui'               => true,
        'show_in_menu'          => true,
        'show_admin_column'     => true,
        'query_var'             => true,
        'rewrite'               => false,
        'show_in_rest'          => true,
        'publicly_queryable'    => false,
        'rest_base'             => 'ticket_product',
        'rest_controller_class' => 'WP_REST_Terms_Controller',
        'update_count_callback' => '_update_generic_term_count',
        'capabilities'          => array(
            'manage_terms' => 'manage_products',
            'edit_terms'   => 'manage_products',
            'delete_terms' => 'manage_products',
            'assign_terms' => 'assign_product'
        )
    ));

    register_taxonomy('fast_agent', array('fast_ticket'), array(
        'labels'                => array(
            'name'              => esc_html_x('Agents', 'taxonomy general name', 'fast-plugin'),
            'singular_name'     => esc_html_x('Agent', 'taxonomy singular name', 'fast-plugin'),
            'search_items'      => esc_html__('Search Agents', 'fast-plugin'),
            'all_items'         => esc_html__('All Agents', 'fast-plugin'),
            'parent_item'       => esc_html__('Parent Agent', 'fast-plugin'),
            'parent_item_colon' => esc_html__('Parent Agent:', 'fast-plugin'),
            'edit_item'         => esc_html__('Edit Agent', 'fast-plugin'),
            'update_item'       => esc_html__('Update Agent', 'fast-plugin'),
            'add_new_item'      => esc_html__('Add New Agent', 'fast-plugin'),
            'new_item_name'     => esc_html__('New Agent Name', 'fast-plugin'),
            'menu_name'         => esc_html__('Agents', 'fast-plugin'),
        ),
        'hierarchical'          => false,
        'show_ui'               => true,
        'show_in_menu'          => true,
        'show_admin_column'     => true,
        'query_var'             => true,
        'rewrite'               => false,
        'show_in_rest'          => true,
        'publicly_queryable'    => false,
        'rest_base'             => 'ticket_agent',
        'rest_controller_class' => 'WP_REST_Terms_Controller',
        'update_count_callback' => '_update_generic_term_count',
        'capabilities'          => array(
            'manage_terms' => 'manage_agents',
            'edit_terms'   => 'manage_agents',
            'delete_terms' => 'manage_agents',
            'assign_terms' => 'assign_agent'
        )
    ));

    // Register User ID meta data for agent taxonomy.
    register_meta('fast_agent', '_user_id', array(
        'sanitize_callback' => 'sanitize_text_field',
        'type'              => 'integer',
        'description'       => esc_html__('User id for this agent.', 'fast-plugin'),
        'single'            => true
    ));

    // Register User ID meta data for agent taxonomy.
    register_meta('fast_product', '_agent_ids', array(
        'sanitize_callback' => 'sanitize_text_field',
        'type'              => 'string',
        'description'       => esc_html__('Users that will handle this product.', 'fast-plugin'),
        'single'            => true
    ));
}
add_action('init', 'fast_register_custom_taxonomies');

/**
 * Allows ticket endpoints for pages & root if front page set to support page
 */
function fast_rewrite_app_endpoints()
{
    $front_page = (int) get_option('page_on_front');
    $support_page_id = (int) fast_get_option('support_page');
    $app_endpoints = array(
        'ticket',
        'login',
        'register',
        'form-builder',
        'envato'
    );

    if ($front_page === $support_page_id) {
        // Support page set as frontpage.
        foreach ($app_endpoints as $endpoint) {
            add_rewrite_rule(
                $endpoint . '(/(.*))?/?$',
                'index.php?ticket=$matches[2]&page_id=' . $support_page_id,
                'top'
            );
        }
        add_filter('redirect_canonical', 'fast_disable_canonical_redirect_for_front_page');
    } else {
        // Support page set as regular page.
        foreach ($app_endpoints as $endpoint) {
            add_rewrite_endpoint($endpoint, EP_PAGES);
        }
    }
}
add_action('init', 'fast_rewrite_app_endpoints');

/*
 * after_setup_theme
 */

/**
 * Adds the options & metaboxes defaults
 */
function fast_create_default_options()
{
    if (class_exists('FastOptionsDefaults')) {
        // Option page defaults.
        $plugin_options = apply_filters('fast_filter_options', array());
        $options_defaults = FastOptionsDefaults::instance();
        foreach ($plugin_options as $option_page) {
            foreach ($option_page['sections'] as $section) {
                foreach ($section['fields'] as $field) {
                    if (isset($field['id']) && isset($field['default'])) {
                        $options_defaults->add_default('fast-plugin-options', $field['id'], $field['default']);
                    }
                }
            }
        }
        // Register metabox defaults.
        $metabox_options = apply_filters('fast_filter_metabox_options', array());
        foreach ($metabox_options as $section) {
            foreach ($section['fields'] as $field) {
                if (isset($field['id']) && isset($field['default'])) {
                    $options_defaults->add_default('fast-plugin-metabox-options', $field['id'], $field['default']);
                }
            }
        }
    }
}
add_action('after_setup_theme', 'fast_create_default_options');

/*
 * template_redirect
 */

/**
 * Redirects support page to app
 */
function fast_redirect_template_to_app()
{
    global $wp_query;
    $support_page_id = (int) fast_get_option('support_page');
    if (get_the_ID() === $support_page_id) {
        require FAST_PLUGIN_DIR . 'classes/class-ticket-app.php';
        FastTicketApp::instance();
    }
}
add_action('template_redirect', 'fast_redirect_template_to_app');

/*
 * set_auth_cookie
 */

/**
 * Make sure that we set the superglobal $_COOKIE when logging in so that the API returns a still valid nonce.
 *
 * @param object $auth_cookie Cookie that has been created.
 */
function fast_set_auth_cookie($auth_cookie)
{
    $_COOKIE[AUTH_COOKIE] = $auth_cookie;
}
add_action('set_auth_cookie', 'fast_set_auth_cookie', 10, 1);

/*
 * set_logged_in_cookie
 */

/**
 * Make sure that we set the superglobal $_COOKIE when logging in so that the API returns a still valid nonce.
 *
 * @param object $logged_in_cookie Cookie that has been created.
 */
function fast_set_logged_in_cookie($logged_in_cookie)
{
    $_COOKIE[LOGGED_IN_COOKIE] = $logged_in_cookie;
}
add_action('set_logged_in_cookie', 'fast_set_logged_in_cookie', 10, 1);


/*
 * rest_pre_insert_fast_purchase
 */

/**
 * Calls Envato when purchase is added / updated to fetch product information.
 *
 * @param object $prepared_post Post object.
 * @param object $request HTTP Request.
 */
function fast_on_before_create_purchase($prepared_post, $request)
{
    // Get PC from request or post if already exists.
    $purchase_code = isset($prepared_post->ID) ? get_post_meta($prepared_post->ID, '_purchase_code', true) : $request->get_param('purchase_code');

    // If new purchase check if we already have added.
    if (!isset($prepared_post->ID)) {
        if (false !== fast_get_user_envato_purchase($purchase_code, get_current_user_id())) {
            return new WP_Error('fast_purchase_already_exits', esc_html__('You already have added this purchase', 'fast-plugin'), array(
                'status' => 400
            ));
        }
    }

    // Get purchase from Envato.
    $purchase = fast_fetch_purchase_information($purchase_code);

    if (is_wp_error($purchase)) {
        return $purchase;
    }

    $prepared_post->meta_input = array(
        '_purchase_code' => $purchase_code
    );

    // Got the purchase data so save it to meta.
    foreach ($purchase as $field => $value) {
        $prepared_post->meta_input['_' . $field] = $value;
    }

    return $prepared_post;
}
add_action('rest_pre_insert_fast_purchase', 'fast_on_before_create_purchase', 10, 2);

/*
 * save_post_fast_ticket
 */

/**
 * Assigns default status to a new ticket
 *
 * @param int  $post_id Post id being saved.
 * @param post $post The post object.
 * @param bool $update Whether this is an existing post being updated or not.
 */
function fast_on_create_new_ticket($post_id, $post, $update)
{
    if ((defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) || wp_is_post_revision($post_id)) {
        return;
    }

    if (true === $update) {
        return;
    }
    // Make sure ticket is open.
    wp_set_object_terms($post_id, 'open', 'fast_status', false);

    // Increase the user's ticket count.
    $current_user_id = get_current_user_id();
    $user_ticket_count = (int) get_user_meta($current_user_id, '_ticket_count', true);
    $updated_ticket_count = $user_ticket_count + 1;
    update_user_meta($current_user_id, '_ticket_count', $updated_ticket_count);
}
add_action('save_post_fast_ticket', 'fast_on_create_new_ticket', 10, 3);

/**
 * Decreases ticket count upon ticket deletion.
 *
 * @param int  $post_id Post id being saved.
 * @param post $post The post object.
 * @param bool $update Whether this is an existing post being updated or not.
 */
function fast_on_delete_ticket($post_id, $post, $update)
{
    if ((defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) || wp_is_post_revision($post_id)) {
        return;
    }

    if (false === $update || 'trash' !== get_post_status($post_id)) {
        return;
    }

    // Decrease user ticket count.
    $current_user_id = get_current_user_id();
    $user_ticket_count = (int) get_user_meta($current_user_id, '_ticket_count', true);
    $updated_ticket_count = $user_ticket_count - 1;
    update_user_meta($current_user_id, '_ticket_count', $updated_ticket_count);
}
add_action('save_post_fast_ticket', 'fast_on_delete_ticket', 10, 3);

/*
 * wp_insert_comment
 */

/**
 * Sends an email and/or slack notice to the agent/client upon comment addition
 *
 * @param int    $comment_id Comment id being saved.
 * @param object $comment Comment being saved.
 */
function fast_notify_on_ticket_reply($comment_id, $comment)
{
    if ('fast_ticket' !== get_post_type($comment->comment_post_ID)) {
        return;
    }

    $ticket          = get_post($comment->comment_post_ID);
    $is_user_agent   = fast_is_user_agent($comment->user_id);
    $sender          = get_userdata($comment->user_id);
    $receiver        = $is_user_agent ? get_userdata($ticket->post_author) : fast_get_ticket_agent($ticket->ID);
    $is_new_ticket   = intval($ticket->comment_count) === 1;

    /* Send Email */

    // Get option for email.
    $email_notify_enabled = $is_new_ticket ? fast_get_option('email_new_ticket') : ($is_user_agent ? fast_get_option('email_agent_reply') : fast_get_option('email_client_reply'));

    // If we have someone to send to and option is enabled.
    if (false !== $receiver && 'on' === $email_notify_enabled) {
        // Get subject & content.
        $email_subject = $is_new_ticket ? fast_get_option('ticket_new_email_subject') : fast_get_option('ticket_reply_email_subject');
        $email_content = $is_new_ticket ? fast_get_option('ticket_new_email_content') : fast_get_option('ticket_reply_email_content');
        // Send email.
        fast_send_ticket_email($sender, $receiver, $email_subject, $email_content, $ticket, $comment);
    }

    /* Send Slack */

    // Get option for slack & hook.
    $slack_notify_enabled = $is_new_ticket ? fast_get_option('slack_new_ticket') : ($is_user_agent ? fast_get_option('slack_agent_reply') : fast_get_option('slack_client_reply'));
    $slack_webhook = fast_get_option('slack_webhook');

    // If we have a hook and slack is enabled.
    if ('on' === $slack_notify_enabled && !empty($slack_webhook)) {
        // Get content.
        $notice_content = $is_new_ticket ? fast_get_option('ticket_new_slack_content') : fast_get_option('ticket_reply_slack_content');
        $stripped_comment = (object) array(
            'comment_content' => wp_kses($comment->comment_content, array())
        );
        // Send slack notification.
        fast_send_slack_notification($sender, $receiver, $notice_content, $ticket, $stripped_comment);
    }
}
add_action('wp_insert_comment', 'fast_notify_on_ticket_reply', 10, 2);

/**
 * Marks ticket if needs a reply upon comment addition
 *
 * @param int    $comment_id Comment id being saved.
 * @param object $comment_object Comment being saved.
 */
function fast_mark_ticket_if_needs_reply($comment_id, $comment_object)
{
    if (get_post_type($comment_object->comment_post_ID) !== 'fast_ticket') {
        return;
    }

    $agent_replied = fast_is_user_agent($comment_object->user_id) === true ? 'true' : 'false';
    update_post_meta($comment_object->comment_post_ID, '_agent_replied', $agent_replied);
}
add_action('wp_insert_comment', 'fast_mark_ticket_if_needs_reply', 11, 2);

/**
 * Marks ticket as open on comment addition
 *
 * @param int    $comment_id Comment id being saved.
 * @param object $comment_object Comment being saved.
 */
function fast_open_ticket_on_comment($comment_id, $comment_object)
{
    if (get_post_type($comment_object->comment_post_ID) !== 'fast_ticket') {
        return;
    }

    $ticket_status = wp_get_post_terms($comment_object->comment_post_ID, 'fast_status');
    if ($ticket_status && !is_wp_error($ticket_status)) {
        $ticket_status = $ticket_status[0];
        if ('closed' === $ticket_status->slug) {
            wp_set_object_terms($comment_object->comment_post_ID, 'open', 'fast_status', false);
        }
    }
}
add_action('wp_insert_comment', 'fast_open_ticket_on_comment', 12, 2);

/**
 * Writes all ticket comments to the excerpt so they may be searched.
 *
 * @param int    $comment_id Comment id being saved.
 * @param object $comment_object Comment being saved.
 */
function fast_update_ticket_excerpt_with_comments($comment_id, $comment_object)
{
    if (get_post_type($comment_object->comment_post_ID) !== 'fast_ticket') {
        return;
    }

    $ticket = get_post($comment_object->comment_post_ID);
    $ticket->post_content = fast_create_ticket_content_from_comments($ticket);
    wp_update_post($ticket);
}
add_action('wp_insert_comment', 'fast_update_ticket_excerpt_with_comments', 13, 2);

/*
 * set_object_terms
 */

/**
 * Sends an email and or slack notice to the agent/client upon comment addition
 *
 * @param int    $object_id  Object ID.
 * @param array  $terms      An array of object terms.
 * @param array  $tt_ids     An array of term taxonomy IDs.
 * @param string $taxonomy   Taxonomy slug.
 * @param bool   $append     Whether to append new terms to the old terms.
 * @param array  $old_tt_ids Old array of term taxonomy IDs.
 */
function fast_set_agent_after_create_new_ticket($object_id, $terms, $tt_ids, $taxonomy, $append, $old_tt_ids)
{
    if ('fast_ticket' !== get_post_type($object_id) || 'fast_product' !== $taxonomy) {
        return;
    }

    $ticket = get_post($object_id);
    if (0 !== (int) $ticket->comment_count) {
        return;
    }

    // loop through the assigned agents for that product, and assign them in a round robin fashion.
    $product_terms = wp_get_post_terms($object_id, 'fast_product');
    if ($product_terms && !is_wp_error($product_terms)) {
        $product_terms = $product_terms[0];
        $agents = fast_get_taxonomy_metabox_value('product_user_ids', $product_terms->term_id);
        if ($agents) {
            wp_set_object_terms($object_id, intval($agents[$product_terms->count % count($agents)]), 'fast_agent', false);
            return;
        }
    }

    // if not product cat, or no agents for that cat, assign the default one.
    $default_agent = fast_get_option('default_agent');
    if (!empty($default_agent)) {
        wp_set_object_terms($object_id, intval($default_agent), 'fast_agent', false);
        return;
    }

    // if no default one, assign a random one.
    $agents = array();
    $agent_terms = get_terms('fast_agent', array(
        'hide_empty' => false,
    ));
    foreach ($agent_terms as $agent) {
        $agents[] = $agent->term_id;
    }
    if (!empty($agents)) {
        wp_set_object_terms($object_id, intval($agents[rand() % count($agents)]), 'fast_agent', false);
    }
}
add_action('set_object_terms', 'fast_set_agent_after_create_new_ticket', 10, 6);

/**
 * Sends an email to the agent/client upon agent assignement.
 *
 * @param int    $object_id  Object ID.
 * @param array  $terms      An array of object terms.
 * @param array  $tt_ids     An array of term taxonomy IDs.
 * @param string $taxonomy   Taxonomy slug.
 * @param bool   $append     Whether to append new terms to the old terms.
 * @param array  $old_tt_ids Old array of term taxonomy IDs.
 */
function fast_notify_after_agent_change($object_id, $terms, $tt_ids, $taxonomy, $append, $old_tt_ids)
{
    if ('fast_ticket' !== get_post_type($object_id) || 'fast_agent' !== $taxonomy) {
        return;
    }

    // Only do this from manual agent assigns.
    $ticket = get_post($object_id);
    if (0 === (int) $ticket->comment_count) {
        return;
    }

    $current_user_id = get_current_user_id();
    $sender          = get_userdata($current_user_id);
    $agent_id        = fast_get_taxonomy_metabox_value('agent_user_id', $terms[0]);
    $receiver        = get_userdata($agent_id);

    if (false !== $receiver && 'on' === fast_get_option('email_ticket_assignment') && (int) $current_user_id !== (int) $agent_id) {
        $email_subject   = fast_get_option('ticket_assign_email_subject');
        $email_content   = fast_get_option('ticket_assign_email_content');
        fast_send_ticket_email($sender, $receiver, $email_subject, $email_content, $ticket, false);
    }
    $slack_webhook = fast_get_option('slack_webhook');
    if (false !== $receiver && !empty($slack_webhook) && 'on' === fast_get_option('slack_ticket_assignment')) {
        $notice_content  = fast_get_option('ticket_assign_slack_content');
        fast_send_slack_notification($sender, $receiver, $notice_content, $ticket, false);
    }
}
add_action('set_object_terms', 'fast_notify_after_agent_change', 10, 6);

/*
 * rest_api_init
 */

/**
 * Registers api class routes.$_COOKIE
 */
function fast_register_extra_api_routes()
{
    include FAST_PLUGIN_DIR . 'classes/class-api.php';
    $api = new FastAPI();
    $api->register_routes();
}
add_action('rest_api_init', 'fast_register_extra_api_routes');

/**
 * Adds post meta fields in the REST GET responses.
 */
function fast_add_post_meta_in_rest()
{
    /**
     * Taxonomy
     */
    register_rest_field('taxonomy', 'singular', array(
        'get_callback' => function ($taxonomy) {
            return $taxonomy['labels']->singular_name;
        },
        'schema' => array(
            'description' => esc_html__('Singular Label', 'fast-plugin'),
            'type'        => 'string'
        ),
    ));

    register_rest_field('taxonomy', 'terms', array(
        'get_callback' => function ($taxonomy) {
            return get_terms(array(
                'taxonomy'   => $taxonomy['slug'],
                'hide_empty' => false,
            ));
        },
        'schema' => array(
            'description' => esc_html__('Terms', 'fast-plugin'),
            'type'        => 'array'
        ),
    ));

    /**
     * Ticket
     */

    register_rest_field('fast_ticket', 'author_email', array(
        'get_callback' => function ($post) {
            return get_the_author_meta('user_email', $post['author']);
        },
        'schema' => array(
            'description' => esc_html__('Author Email Address', 'fast-plugin'),
            'type'        => 'string'
        ),
    ));

    register_rest_field('fast_ticket', 'author_name', array(
        'get_callback' => function ($post) {
            $first_name = get_the_author_meta('first_name', $post['author']);
            $last_name = get_the_author_meta('last_name', $post['author']);
            return trim("$first_name $last_name");
        },
        'schema' => array(
            'description' => esc_html__('Author Display Name', 'fast-plugin'),
            'type'        => 'string'
        ),
    ));

    register_rest_field('fast_ticket', 'author_avatar', array(
        'get_callback' => function ($post) {
            return get_avatar_url($post['author']);
        },
        'schema' => array(
            'description' => esc_html__('Author Avatar', 'fast-plugin'),
            'type'        => 'string'
        ),
    ));

    register_rest_field('fast_ticket', 'customer_waiting', array(
        'get_callback' => function ($post) {
            return get_post_meta($post['id'], '_customer_waiting', true);
        },
        'schema' => array(
            'description' => esc_html__('Indicates if a user needs to reply to a ticket', 'fast-plugin'),
            'type'        => 'boolean'
        ),
    ));

    register_rest_field('fast_ticket', 'agent_replied', array(
        'get_callback' => function ($post) {
            return get_post_meta($post['id'], '_agent_replied', true);
        },
        'schema' => array(
            'description' => esc_html__('Indicates if a user needs to reply to a ticket', 'fast-plugin'),
            'type'        => 'boolean'
        ),
    ));

    register_rest_field('fast_ticket', 'is_open', array(
        'get_callback' => function ($post) {
            $status = get_the_terms($post['id'], 'fast_status');
            if (is_array($status) && count($status) > 0) {
                return 'open' === $status[0]->slug;
            }
            return false;
        },
        'schema' => array(
            'description' => esc_html__('Indicates if a ticket is open or not', 'fast-plugin'),
            'type'        => 'boolean'
        ),
    ));

    register_rest_field('fast_ticket', 'author_ticket_count', array(
        'get_callback' => function ($post) {
            return (int) get_user_meta($post['author'], '_ticket_count', true);
        },
        'schema' => array(
            'description' => esc_html__('Author\'s total number of created tickets', 'fast-plugin'),
            'type'        => 'number'
        ),
    ));

    register_rest_field('fast_ticket', 'capabilities', array(
        'get_callback' => function ($post) {
            $user_id = get_current_user_id();
            return array(
                'assign_status'  => current_user_can('assign_status', $post['id']),
                'assign_agent'   => current_user_can('assign_agent', $post['id']),
                // Assign product only if you are an agent.
                // But permission is needed for customer to assign product when they create a ticket.
                'assign_product' => fast_is_user_agent($user_id),
                'delete_ticket'  => current_user_can('delete_ticket', $post['id']),
            );
        }
    ));

    register_rest_field('fast_ticket', 'envato_purchase_id', array(
        'get_callback' => function ($post) {
            return (int) get_post_meta($post['id'], '_envato_purchase_id', true);
        },
        'update_callback' => function ($value, $post) {
            update_post_meta($post->ID, '_envato_purchase_id', $value);
        },
        'schema' => array(
            'description' => esc_html__('Envato purchase id.', 'fast-plugin'),
            'type'        => 'integer'
        ),
    ));

    register_rest_field('fast_ticket', 'purchase', array(
        'get_callback' => function ($post) {
            $user_id = get_current_user_id();
            if (fast_is_user_agent($user_id)) {
                $envato_purchase_id = get_post_meta($post['id'], '_envato_purchase_id', true);
                if (!empty($envato_purchase_id)) {
                    $purchase_info = get_post_custom($envato_purchase_id);
                    if (!empty($purchase_info)) {
                        return array(
                            'id'              => $envato_purchase_id,
                            'buyer'           => $purchase_info['_buyer'][0],
                            'created_at'      => $purchase_info['_created_at'][0],
                            'item_id'         => $purchase_info['_item_id'][0],
                            'item_name'       => $purchase_info['_item_name'][0],
                            'licence'         => $purchase_info['_licence'][0],
                            'purchase_code'   => $purchase_info['_purchase_code'][0],
                            'supported_until' => $purchase_info['_supported_until'][0],
                            'ticket_product'  => $purchase_info['_ticket_product'][0],
                        );
                    }
                }
            }
        },
        'schema' => array(
            'description' => esc_html__('Envato purchase details.', 'fast-plugin'),
            'type'        => 'object'
        ),
    ));

    register_rest_field('fast_ticket', 'woocommerce_order_id', array(
        'get_callback' => function ($post) {
            return (int) get_post_meta($post['id'], '_woocommerce_order_id', true);
        },
        'update_callback' => function ($value, $post) {
            update_post_meta($post->ID, '_woocommerce_order_id', $value);
        },
        'schema' => array(
            'description' => esc_html__('WooCommerce Order ID.', 'fast-plugin'),
            'type'        => 'integer'
        ),
    ));

    register_rest_field('fast_ticket', 'woocommerce_product_ids', array(
        'get_callback' => function ($post) {
            return get_post_meta($post['id'], '_woocommerce_product_ids', true);
        },
        'update_callback' => function ($value, $post) {
            update_post_meta($post->ID, '_woocommerce_product_ids', $value);
        },
        'schema' => array(
            'description' => esc_html__('WooCommerce Product IDs.', 'fast-plugin'),
            'type'        => 'array',
            'items'       => array(
                'type' => 'integer',
            )
        ),
    ));

    register_rest_field('fast_ticket', 'extra_fields', array(
        'get_callback' => function ($post) {
            return get_post_meta($post['id'], '_extra_fields', true);
        },
        'update_callback' => function ($value, $post) {
            update_post_meta($post->ID, '_extra_fields', $value);
        },
        'schema' => array(
            'description' => esc_html__('Extra ticket information.', 'fast-plugin'),
            'type'        => 'object'
        ),
    ));

    register_rest_field('fast_ticket', 'from_email', array(
        'get_callback' => function ($post) {
            $from_email = get_post_meta($post['id'], '_from_email', false);
            return !empty($from_email);
        },
        'schema' => array(
            'description' => esc_html__('Is this ticket from email.', 'fast-plugin'),
            'type'        => 'boolean'
        ),
    ));

    /**
     * Comment
     */

    register_rest_field('comment', 'attachments', array(
        'get_callback' => function ($comment) {
            $attachment_meta = get_comment_meta($comment['id'], '_attachments', true);
            if (!empty($attachment_meta)) {
                if (is_array($attachment_meta)) {
                    return $attachment_meta;
                }
                // For versions older that 1.3
                // Some names might be badly encoded in a non-standard way. Change encoding notation and convert it to UTF-8.
                $attachments = html_entity_decode(preg_replace('/u+([0-9A-F]{4})/', "&#x\\1;", $attachment_meta), ENT_NOQUOTES, 'UTF-8');
                return json_decode($attachments, true);
            }
            return array();
        },
        'update_callback' => function ($value, $comment) {
             update_comment_meta($comment->comment_ID, '_attachments', $value);
        }
    ));

    register_rest_field('comment', 'from_email', array(
        'get_callback' => function ($comment) {
            $from_email = get_comment_meta($comment['id'], '_from_email', false);
            return !empty($from_email);
        },
        'schema' => array(
            'description' => esc_html__('Is this comment from email.', 'fast-plugin'),
            'type'        => 'boolean'
        ),
    ));

    /**
     * Purchase
     */

    register_rest_field('fast_purchase', 'purchase_code', array(
        'get_callback' => function ($post) {
            return get_post_meta($post['id'], '_purchase_code', true);
        },
        'update_callback' => function ($value, $post) {
            update_post_meta($post->ID, '_purchase_code', $value);
        },
        'schema' => array(
            'description' => esc_html__('Envato purchase code.', 'fast-plugin'),
            'type'        => 'string'
        ),
    ));

    register_rest_field('fast_purchase', 'item_name', array(
        'get_callback' => function ($post) {
            return get_post_meta($post['id'], '_item_name', true);
        },
        'schema' => array(
            'description' => esc_html__('Envato Item Name', 'fast-plugin'),
            'type'        => 'string'
        ),
    ));

    register_rest_field('fast_purchase', 'item_id', array(
        'get_callback' => function ($post) {
            return get_post_meta($post['id'], '_item_id', true);
        },
        'schema' => array(
            'description' => esc_html__('Envato Item ID', 'fast-plugin'),
            'type'        => 'string'
        ),
    ));

    register_rest_field('fast_purchase', 'created_at', array(
        'get_callback' => function ($post) {
            return get_post_meta($post['id'], '_created_at', true);
        },
        'schema' => array(
            'description' => esc_html__('Envato Created At', 'fast-plugin'),
            'type'        => 'string'
        ),
    ));

    register_rest_field('fast_purchase', 'buyer', array(
        'get_callback' => function ($post) {
            return get_post_meta($post['id'], '_buyer', true);
        },
        'schema' => array(
            'description' => esc_html__('Envato Buyer', 'fast-plugin'),
            'type'        => 'string'
        ),
    ));

    register_rest_field('fast_purchase', 'licence', array(
        'get_callback' => function ($post) {
            return get_post_meta($post['id'], '_licence', true);
        },
        'schema' => array(
            'description' => esc_html__('Envato Licence', 'fast-plugin'),
            'type'        => 'string'
        ),
    ));

    register_rest_field('fast_purchase', 'supported_until', array(
        'get_callback' => function ($post) {
            return get_post_meta($post['id'], '_supported_until', true);
        },
        'schema' => array(
            'description' => esc_html__('Envato Supported Until', 'fast-plugin'),
            'type'        => 'string'
        ),
    ));

    register_rest_field('fast_purchase', 'ticket_product', array(
        'get_callback' => function ($post) {
            return (int) get_post_meta($post['id'], '_ticket_product', true);
        },
        'schema' => array(
            'description' => esc_html__('Fast ticket product id', 'fast-plugin'),
            'type'        => 'integer'
        ),
    ));
}
add_action('rest_api_init', 'fast_add_post_meta_in_rest');

/**
 * Add the necessary filter to tickets post type requests
 */
function fast_rest_api_add_ticket_filter()
{
    add_filter('rest_fast_ticket_query', 'fast_rest_api_filter_add_filter_param', 10, 2);
}
add_action('rest_api_init', 'fast_rest_api_add_ticket_filter');

/**
 * Adds a filter options as response header.
 *
 * @param boolean $served Indicates if the request has already been served.
 * @param object  $response Response Object.
 * @param object  $request Request Object.
 */
function fast_rest_api_add_filters_in_headers($served, $response, $request)
{
    if ($request->get_method() === 'GET' && $request->get_route() === '/wp/v2/tickets' && null !== $request->get_param('filter')) {
        $user_id        = get_current_user_id();
        $is_agent       = fast_is_user_agent($user_id);
        $filters        = $is_agent ? apply_filters('fast_filter_get_agent_filters', $user_id) : apply_filters('fast_filter_get_client_filters', $user_id);
        $request_filter = $request->get_param('filter');
        $pages = $request->get_param('pages');
        $pages = empty($pages) ? 1 : $pages;

        $returned_filters = array();
        foreach ($filters as $key => $filter) {
            $returned_filter = array();
            $returned_filter['name'] = $filter['name'];
            $returned_filter['id'] = $filter['id'];
            $returned_filter['count'] = isset($filter['count']) ? $filter['count'] : fast_get_query_count($filter['args']);
            $returned_filter['needReplyCount'] = fast_get_need_reply_count($filter['args'], $is_agent);
            if ($request_filter === $filter['id']) {
                $returned_filter['active'] = true;
                $returned_filter['pages'] = $pages;
            }
            $returned_filters[] = $returned_filter;
        }
        header('ticket-filters:' . wp_json_encode($returned_filters));
    }
    return $served;
}

/**
 * Adds a filter that modifies response headers.
 */
function fast_rest_api_add_headers()
{
    add_filter('rest_pre_serve_request', 'fast_rest_api_add_filters_in_headers', 10, 3);
}
add_action('rest_api_init', 'fast_rest_api_add_headers');

/*
 * fast_action_demo_cleanup
 */

/**
 * Removes all comments and tickets created after a date (for demo site)
 *
 * @param string $date Date for searching after.
 */
function fast_demo_cleanup_content($date = '')
{
    $date_query = array(
        'after' => $date,
        'inclusive' => true,
    );
    $new_comments = get_comments(array(
        'date_query' => $date_query
    ));
    foreach ($new_comments as $comment) {
        wp_delete_comment($comment->comment_ID, true);
    }
    $new_tickets = get_posts(array(
        'post_type'      => 'fast_ticket',
        'posts_per_page' => -1,
        'post_status'    => 'publish,private',
        'date_query'     => $date_query
    ));

    foreach ($new_tickets as $ticket) {
        wp_delete_post($ticket->ID, true);
    }
}
add_action('fast_action_demo_cleanup', 'fast_demo_cleanup_content', 10, 1);

/*
 * do_feed_rss2
 * do_feed_atom
 */

/**
 * Removes all comments from feeds
 *
 * @param boolean $for_comments If comment feed.
 */
function fast_remove_comment_feeds($for_comments)
{
    if ($for_comments) {
        remove_action('do_feed_rss2', 'do_feed_rss2', 10, 1);
        remove_action('do_feed_atom', 'do_feed_atom', 10, 1);
    }
}
add_action('do_feed_rss2', 'fast_remove_comment_feeds', 9, 1);
add_action('do_feed_atom', 'fast_remove_comment_feeds', 9, 1);

/**
 * Loads the email pipe class because we need to check for emails.
 */
function fast_check_email_pipe()
{
    // Load IMAP reader.
    require FAST_PLUGIN_DIR . 'lib/imap/Mailbox.php';
    require FAST_PLUGIN_DIR . 'lib/imap/IncomingMail.php';
    // Load Email Parser.
    require FAST_PLUGIN_DIR . 'lib/email-reply-parser/Parser/EmailParser.php';
    require FAST_PLUGIN_DIR . 'lib/email-reply-parser/Parser/FragmentDTO.php';
    require FAST_PLUGIN_DIR . 'lib/email-reply-parser/Email.php';
    require FAST_PLUGIN_DIR . 'lib/email-reply-parser/EmailReplyParser.php';
    require FAST_PLUGIN_DIR . 'lib/email-reply-parser/Fragment.php';
    // Load our email pipe class.
    require FAST_PLUGIN_DIR . 'classes/class-email-pipe.php';
    $email_pipe = new FastEmailPipe();
    // Check emails.
    $email_pipe->check_emails();
}
add_action('fast_action_email_pipe', 'fast_check_email_pipe', 10, 0);
