<?php
/**
 * Admin only actions
 *
 * @package Fast Plugin
 *
 * @copyright 2017 Oxygenna.com
 * @license http://themeforest.net/licenses/standard
 * @version 1.5.0
 * @author Oxygenna
 */

 /*
 * plugins_loaded
 */

 /*
 * init
 */

 /**
  * Checks if updates need to be applied
  */
function fast_check_update()
{
    $plugin_version = get_option('fast_version', '1.0.0');
    if ('1.5.0' !== $plugin_version) {
        // We need to update.
        if (version_compare($plugin_version, '1.2.0', '<')) {
            fast_update_less_than_1_2_0();
        }
        if (version_compare($plugin_version, '1.3.0', '<')) {
            fast_update_less_than_1_3_0();
        }
        update_option('fast_version', '1.5.0');
    }
}
add_action('init', 'fast_check_update');

/**
 * Makes sure all tickets use true / false as strings for the agent replied metadaata
 */
function fast_update_less_than_1_2_0()
{
    $all_tickets = get_posts(array(
        'post_type'     => 'fast_ticket',
        'posts_per_page' => -1,
        'post_status' => 'any'
    ));

    foreach ($all_tickets as $ticket) {
        // Update agent_replied post meta.
        $agent_replied = get_post_meta($ticket->ID, '_agent_replied', true);
        $new_agent_replied = '1' === $agent_replied ||  'true' === $agent_replied ? 'true' : 'false';
        update_post_meta($ticket->ID, '_agent_replied', $new_agent_replied);

        // Save all comments to excerpt so search will work.
        $ticket->post_content = fast_create_ticket_content_from_comments($ticket);
        $new_id = wp_update_post($ticket);
        $updated_post = get_post($new_id);
    }
    // Finally recount all taxonomies to include private tickets.
    fast_recalculate_all_taxonomy_counts();
}

/**
 * Moves customer purchases from user meta to cpt & copys site urls
 */
function fast_update_less_than_1_3_0()
{
    // Copy all siteurl params into new extra field.
    $all_tickets = get_posts(array(
        'post_type'      => 'fast_ticket',
        'posts_per_page' => -1,
        'post_status'    => 'any'
    ));

    foreach ($all_tickets as $ticket) {
        $site_url = get_post_meta($ticket->ID, '_site_url', true);
        if (!empty($site_url)) {
            $extra_fields = get_post_meta($ticket->ID, '_extra_fields', true);
            $extra_fields = empty($extra_fields) ? array() : $extra_fields;
            $extra_fields['site-url'] = $site_url;
            update_post_meta($ticket->ID, '_extra_fields', $extra_fields);
        }
    }
}


/*
 * after_setup_theme
 */

/**
 * Registers a custom post types.
 */
function fast_add_plugin_options_page()
{
    // Add stack options to appearance menu.
    if (class_exists('FastOptions')) {
        // Create new option set with post type as menu.
        $options = new FastOptions('fast-plugin-option_group', 'fast-plugin-options');
        // Load the stack options and add them to the options page.
        $option_pages = apply_filters('fast_filter_options', array());
        foreach ($option_pages as $options_page) {
            $options->add_menu_page($options_page);
        }
    }
}
add_action('after_setup_theme', 'fast_add_plugin_options_page');

if (!function_exists('fast_setup_metaboxes')) {
    /**
     * Loads and sets up the theme metaboxes.
     *
     * @return void
     **/
    function fast_setup_metaboxes()
    {
        if (class_exists('FastMetaboxes')) {
            $metaboxes = new FastMetaboxes();
            $metabox_options = apply_filters('fast_filter_metabox_options', array());
            foreach ($metabox_options as $metabox) {
                $metaboxes->add_metabox($metabox);
            }
        }
    }
    add_action('after_setup_theme', 'fast_setup_metaboxes');
}

/*
 * init
 */

/**
 * Checks if we are a customer and if so hides all admin pages except the profile page.
 */
function fast_is_customer_allowed()
{
    $current_user = wp_get_current_user();
    if (in_array('fast_customer', (array) $current_user->roles)) {
        add_action('admin_init', 'fast_lock_customer_dashboard_redirect');
        add_action('admin_head', 'fast_lock_customer_hide_side_menus');
        add_action('admin_bar_menu', 'fast_lock_customer_hide_menus', 999);
    }
}
add_action('init', 'fast_is_customer_allowed');

/*
 * admin_init
 */

/**
 * Redirects all admin page requests to the profile page.
 */
function fast_lock_customer_dashboard_redirect()
{
    global $pagenow;

    if ('profile.php' != $pagenow) {
        wp_redirect(admin_url('profile.php'));
        exit;
    }
}

/*
 * admin_head
 */

/**
 * Removes all admin side menus except profile
 */
function fast_lock_customer_hide_side_menus()
{
    global $menu;

    $menu_ids = array();

    // Gather menu IDs (minus profile.php).
    foreach ($menu as $index => $values) {
        if (isset($values[2])) {
            if ('profile.php' == $values[2]) {
                continue;
            }

            // Remove menu pages.
            remove_menu_page($values[2]);
        }
    }
}

/*
 * admin_bar_menu
 */

/**
 * Removes top admin bar menu items when logged in as customer.
 *
 * @param object $wp_admin_bar Admin bar menu object.
 */
function fast_lock_customer_hide_menus($wp_admin_bar)
{
    $all_toolbar_nodes = $wp_admin_bar->get_nodes();
    $allowed_toolbar_node_ids = apply_filters('fast_filter_customer_allowed_admin_menu_nodes', array());

    foreach ($all_toolbar_nodes as $node) {
        if (!in_array($node->id, $allowed_toolbar_node_ids)) {
            $wp_admin_bar->remove_menu($node->id);
        }
    }
}

/*
 * fast_action_register_options_menu_fast-plugin-general-options
 */

/**
 * Creates the plugin admin menu
 *
 * @param object $menu The menu FastMenuPage object.
 */
function fast_add_plugin_general_options_menu($menu)
{
    $menu->hook = add_menu_page($menu->options['page']['title'], $menu->options['menu']['title'], 'manage_options', $menu->options['slug'], array($menu, 'render_option_page'), 'dashicons-sos');
}
add_action('fast_action_register_options_menu_fast-plugin-general-options', 'fast_add_plugin_general_options_menu', 10, 1);

/*
 * fast_action_register_options_menu_fast-plugin-roles-capabilities
 */

/**
 * Creates the plugin admin menu
 *
 * @param object $menu The menu FastMenuPage object.
 */
function fast_add_plugin_roles_options_menu($menu)
{
    $menu->hook = add_submenu_page('fast-plugin-general-options', $menu->options['page']['title'], $menu->options['menu']['title'], 'manage_options', $menu->options['slug'], array($menu, 'render_option_page'));
}
add_action('fast_action_register_options_menu_fast-plugin-registration-page', 'fast_add_plugin_roles_options_menu', 10, 1);
add_action('fast_action_register_options_menu_fast-plugin-login-page', 'fast_add_plugin_roles_options_menu', 10, 1);
add_action('fast_action_register_options_menu_fast-plugin-new-ticket-page', 'fast_add_plugin_roles_options_menu', 10, 1);
add_action('fast_action_register_options_menu_fast-plugin-roles-capabilities', 'fast_add_plugin_roles_options_menu', 10, 1);
add_action('fast_action_register_options_menu_fast-plugin-email-options', 'fast_add_plugin_roles_options_menu', 10, 1);
add_action('fast_action_register_options_menu_fast-plugin-advanced', 'fast_add_plugin_roles_options_menu', 10, 1);
add_action('fast_action_register_options_menu_fast-plugin-slack', 'fast_add_plugin_roles_options_menu', 10, 1);
add_action('fast_action_register_options_menu_fast-plugin-envato', 'fast_add_plugin_roles_options_menu', 10, 1);
add_action('fast_action_register_options_menu_fast-plugin-extra-fields', 'fast_add_plugin_roles_options_menu', 10, 1);
add_action('fast_action_register_options_menu_fast-plugin-woocommerce', 'fast_add_plugin_roles_options_menu', 10, 1);
add_action('fast_action_register_options_menu_fast-plugin-email-pipe', 'fast_add_plugin_roles_options_menu', 10, 1);
add_action('fast_action_register_options_menu_fast-plugin-advanced', 'fast_add_plugin_roles_options_menu', 10, 1);

/*
 * fast_action_option_page_saved_fast-plugin-general-options
 */

/**
 * Make sure permalinks work after saving support page option.
 *
 * @param array $data New options data.
 */
function fast_flush_rewrite_rules_after_save_options($data)
{
    flush_rewrite_rules();
}
add_action('fast_action_option_page_saved_fast-plugin-general-options', 'fast_flush_rewrite_rules_after_save_options', 10, 1);

/*
 * fast_action_option_page_saved_fast-plugin-roles-capabilities
 */

/**
 * Make sure roles are saved after saving roles options.
 *
 * @param mixed $data New data being stored.
 */
function fast_recreate_roles_after_save_roles_option_page($data)
{
    $options_defaults = FastOptionsDefaults::instance();
    $options = wp_parse_args($data, $options_defaults->get_defaults('fast-plugin-options'));

    $roles = apply_filters('fast_filter_ticket_roles', array());
    foreach ($roles as $role => $name) {
        $saved_capabilities = $options[$role . '_capabilities'];
        $capabilities = array();
        foreach ($saved_capabilities as $capability) {
            $capabilities[$capability] = true;
        }
        remove_role($role);
        add_role($role, $name, $capabilities);
    }

    // Add all manager capabilities to administrator.
    $administrator = get_role('administrator');
    $manager_caps  = apply_filters('fast_filter_default_fast_manager_capabilities', array());
    foreach ($manager_caps as $cap) {
        $administrator->add_cap($cap);
    }
}
add_action('fast_action_option_page_saved_fast-plugin-roles-capabilities', 'fast_recreate_roles_after_save_roles_option_page', 10, 1);

/*
 * fast_action_option_page_saved_fast-plugin-email-pipe
 */

/**
 * When email piping options page is saved we schedule or unschedule the chron hook.
 *
 * @param mixed $data New data being stored.
 */
function fast_schedule_email_pipe_if_needed($data)
{
    // Get options.
    $options_defaults = FastOptionsDefaults::instance();
    $options = wp_parse_args($data, $options_defaults->get_defaults('fast-plugin-options'));

    // Check if timespan has changed, if so remove old scheduled event.
    // Also remove if we just turned off piping.
    if (fast_get_option('emailp_timespan') !== $options['emailp_timespan'] || 'off' === $options['emailp_enabled']) {
        wp_clear_scheduled_hook('fast_action_email_pipe');
    }

    if ('on' === $options['emailp_enabled']) {
        // Schedule if we need to.
        if (false === wp_next_scheduled('fast_action_email_pipe')) {
            wp_schedule_event(time(), $options['emailp_timespan'], 'fast_action_email_pipe');
        }
    }

    // Check if someone clicked the run now button.
    if (isset($_POST['emailp_run_check'])) {
        // Clear schedule and start an email check now.
        wp_clear_scheduled_hook('fast_action_email_pipe');
        wp_schedule_event(time(), $options['emailp_timespan'], 'fast_action_email_pipe');
    }
}
add_action('fast_action_option_page_saved_fast-plugin-email-pipe', 'fast_schedule_email_pipe_if_needed', 10, 1);

/**
 * Make sure permalinks work after saving support page option.
 *
 * @param array $data New options data.
 */
function fast_load_envato_products_from_api($data)
{
    if (isset($_POST['envato_products_load'])) {
        $page_size = 100;
        $first_page_items = fast_generate_envato_get_marketplace_items(fast_get_option('envato_username'), $page_size, 1);

        if (!is_wp_error($first_page_items)) {
            $first_page_items = json_decode(wp_remote_retrieve_body($first_page_items), true);

            if (isset($first_page_items['matches'])) {
                // Save results.
                $products = fast_add_envato_products($first_page_items);

                // If we have more than 100 products make additional calls.
                if ($first_page_items['total_hits'] > $page_size) {
                    $pages_to_fetch = ($first_page_items['total_hits'] / $page_size) + 1;
                    for ($page = 2; $page < $pages_to_fetch; $page++) {
                        $extra_page_items = fast_generate_envato_get_marketplace_items(fast_get_option('envato_username'), $page_size, $page);
                        if (!is_wp_error($extra_page_items)) {
                            $extra_page_items = json_decode(wp_remote_retrieve_body($extra_page_items), true);
                            $products = fast_add_envato_products($extra_page_items, $products);
                        }
                    }
                }
                update_option('fast-pluginenvato_product_list', $products);
            }
        }
    }
}
add_action('fast_action_option_page_saved_fast-plugin-envato', 'fast_load_envato_products_from_api', 10, 1);

/**
 * Outputs the extra fields options above the options fields.
 */
function fast_render_custom_fields()
{
    $extra_fields    = fast_get_extra_fields();
    $support_page_id = fast_get_option('support_page');
    $builder_link    = set_url_scheme(trailingslashit(get_the_permalink($support_page_id)) . 'form-builder');
    include FAST_PLUGIN_DIR . 'templates/options/extra-fields.php';
}
add_action('fast-plugin-extra-fields-before-option-fields', 'fast_render_custom_fields', 10, 0);
