<?php
/**
 * Admin only filters
 *
 * @package Fast Plugin
 *
 * @copyright 2017 Oxygenna.com
 * @license http://themeforest.net/licenses/standard
 * @version 1.5.0
 * @author Oxygenna
 */

/**
 * Gets all users that are valid ticket agents
 *
 * @param array $values Array of user id => username.
 */
function fast_agent_taxonomy_user_select_option_values($values)
{
    $values = array();

    $valid_users = get_users(array(
        'role__in' => array(
            'fast_agent',
            'fast_manager',
            'administrator'
        )
    ));

    foreach ($valid_users as $user) {
        $values[$user->ID] = $user->display_name;
    }

    return $values;
}
add_filter('fast_filter_taxonomy_user_select_option_values', 'fast_agent_taxonomy_user_select_option_values');

/**
 * Gets all ticket agents.
 *
 * @param array $values Array of agent id => Agent Name.
 */
function fast_product_agent_ids_values($values)
{
    $values = array();
    $agent_terms = get_terms('fast_agent', array(
        'hide_empty' => false,
    ));
    foreach ($agent_terms as $agent) {
        $values[$agent->term_id] = $agent->name;
    }
    return $values;
}
add_filter('fast_filter_product_agent_ids_values', 'fast_product_agent_ids_values');

/**
 * Gets all ticket products.
 *
 * @param array $values Array of product id => Product Name.
 */
function fast_product_product_ids_values($values)
{
    $values = array();
    $agent_terms = get_terms('fast_product', array(
        'hide_empty' => false,
    ));
    foreach ($agent_terms as $product) {
        $values[$product->term_id] = $product->name;
    }
    return $values;
}
add_filter('fast_filter_product_product_ids_values', 'fast_product_product_ids_values');

/**
 * Adds marketplace item select box to options if envato_products is on
 *
 * @param array $options Plugin meabox options.
 */
function fast_add_envato_product_field_metabox($options)
{
    if ('on' === fast_get_option('envato_products') && isset($options['product_fields'])) {
        array_push($options['product_fields']['fields'], array(
            'name'             => esc_html__('Envato Marketplace Item', 'fast-plugin'),
            'id'               => 'product_envato_item',
            'desc'             => esc_html__('Link this ticket product to an Envato marketplace item.', 'fast-plugin'),
            'type'             => 'select',
            'empty_label'      => esc_html__('No Item Selected', 'fast-plugin'),
            'filter'           => 'fast_filter_product_marketplace_items',
            'select2'          => true,
            'default'          => 0,
            'attributes'       => array(
                'placeholder' => esc_html__('Select a marketplace item', 'fast-plugin'),
                'class'       => 'fast-select2-option',
            )
        ));
    }
    return $options;
}
add_filter('fast_filter_metabox_options', 'fast_add_envato_product_field_metabox', 1, 11);

/**
 * Gets all Envato Marketplace items
 *
 * @param array $items Array of items id => name.
 */
function fast_product_marketplace_items($items)
{
    return get_option('fast-pluginenvato_product_list', array());
}
add_filter('fast_filter_product_marketplace_items', 'fast_product_marketplace_items', 1, 11);

/**
 * Menus that are allowed to be shown in the admin menu when logged in as a customer
 *
 * @param array $node_ids IDs of menu items to allow.
 */
function fast_customer_allowed_admin_menu_nodes($node_ids)
{
    return array(
        'top-secondary',
        'logout',
        'user-info',
        'user-actions',
        'my-account',
        'edit-profile'
    );
}
add_filter('fast_filter_customer_allowed_admin_menu_nodes', 'fast_customer_allowed_admin_menu_nodes');

/**
 * Adds custom columns in Agents taxonomy admin table.
 *
 * @param array $columns Default columns displayed.
 */
function fast_custom_agent_columns($columns)
{
    $columns['_user_id'] = esc_html__('Wordpress User');
    return $columns;
}
add_filter('manage_edit-fast_agent_columns', 'fast_custom_agent_columns');

/**
 * Adds content for custom columns in Agents taxonomy admin table.
 *
 * @param string $content Default content for displayed column.
 * @param string $column_name Column name.
 * @param int    $term_id ID of the term displayed.
 */
function fast_custom_agent_columns_content( $content, $column_name, $term_id )
{
    if ('_user_id' === $column_name) {
        $agent_id = fast_get_taxonomy_metabox_value('agent_user_id', $term_id);
        $agent = get_userdata($agent_id);
        $content = $agent->user_nicename;
    }
    return $content;
}
add_filter('manage_fast_agent_custom_column', 'fast_custom_agent_columns_content', 10, 3);

/**
 * Adds custom columns in Products taxonomy admin table.
 *
 * @param array $columns Default columns displayed.
 */
function fast_custom_product_columns($columns)
{
    $columns['_agent_ids'] = esc_html__('Product Agents');
    return $columns;
}
add_filter('manage_edit-fast_product_columns', 'fast_custom_product_columns');

/**
 * Adds content for custom columns in Products taxonomy admin table.
 *
 * @param string $content Default content for displayed column.
 * @param string $column_name Column name.
 * @param int    $term_id ID of the term displayed.
 */
function fast_custom_product_columns_content( $content, $column_name, $term_id )
{
    if ('_agent_ids' === $column_name) {
        $agents = array();
        $agent_ids= fast_get_taxonomy_metabox_value('product_user_ids', $term_id);
        foreach ($agent_ids as $key => $agent_id) {
            $agent = get_term_by('id', $agent_id, 'fast_product');
            if (false !== $agent) {
                $agents[] = $agent->name;
            }
        }
        $content = implode(', ', $agents);
    }
    return $content;
}
add_filter('manage_fast_product_custom_column', 'fast_custom_product_columns_content', 10, 3);

/*
 * admin_footer_text
 */

/**
 * Changes the admin footer text in plugin pages.
 *
 * @param string $footer_text Default wordpress admin footer text.
 *
 * @return string Modified footer text.
 */
function fast_admin_footer_text($footer_text)
{
    $plugin_pages = apply_filters('fast_filter_options', array());
    $page_names = array();
    foreach ($plugin_pages as $page) {
        $page_names[] = $page['slug'];
    }

    if (isset($_GET['page']) && in_array(wp_unslash($_GET['page']), $page_names)) {
        $footer_text = esc_html__('Fast Plugin - Version 1.5.0', 'fast-plugin');
    }
    return $footer_text;
}
add_filter('admin_footer_text', 'fast_admin_footer_text', 1);

/**
 * Filter for email pipe next schedule on option page.
 *
 * @param string $text Original text.
 */
function fast_get_next_email_pipe_run($text)
{
    $timestamp = wp_next_scheduled('fast_action_email_pipe');
    if (false === $timestamp) {
        $text = esc_html__('No Email Check Scheduled', 'fast-plugin');
    } else {
        $text = '<p>' . esc_html__('Current System Time', 'fast-plugin') . ': ' . get_date_from_gmt(date('Y-m-d H:i:s', time()), 'F j, Y H:i:s') . '</p>';
        $text .= '<p>' . esc_html__('Next Email Check At', 'fast-plugin') . ': ' . get_date_from_gmt(date('Y-m-d H:i:s', $timestamp), 'F j, Y H:i:s') . '</p>';
    }
    return $text;
}
add_filter('fast_filter_email_pipe_next_run', 'fast_get_next_email_pipe_run', 10, 1);

/**
 * Filter for email pipe results on option page.
 *
 * @param string $text Original text.
 */
function fast_get_next_email_pipe_results($text)
{
    $report = get_option('fast_email_pipe_report', false);
    if (false !== $report) {
        $text .= '<p><strong>Last Run Report</strong></p>';
        $text .= '<p><strong>Date: </strong>' . get_date_from_gmt(date('Y-m-d H:i:s', $report->date), 'Y-m-d H:i:s') . '</p>';
        $text .= '<p><strong>Parsed Email Count: </strong>' . $report->email_count . '</p>';
        $text .= '<p><strong>Replies Created: </strong>' . $report->replies_created . '</p>';
        $text .= '<p><strong>New Tickets Created: </strong>' . $report->new_tickets_created . '</p>';
        if (count($report->errors) > 0) {
            $text .= '<p><strong>Errors: </strong></p>';
            foreach ($report->errors as $error) {
                $text .= '<p>' . $error . '</p>';
            }
        }
    } else {
        return esc_html__('No Email Check Has Run', 'fast-plugin');
    }
    return $text;
}
add_filter('fast_filter_email_pipe_results', 'fast_get_next_email_pipe_results', 10, 1);

/**
 * Filter for email pipe schedule select box.
 */
function fast_get_cron_schedules()
{
    $schedules = wp_get_schedules();
    $spans = array();
    foreach ($schedules as $value => $span) {
        $spans[$value] = $span['display'];
    }
    return $spans;
}
add_filter('fast_filter_cron_time_spans', 'fast_get_cron_schedules', 10, 0);


/**
 * Filters stored password so we don't store in plaintext.
 *
 * @param string $password Pass to encrypt.
 * @return string Encrypted password.
 **/
function fast_sanitize_password($password)
{
    // Only encrypt if password has changed.
    if (fast_get_option('emailp_password') !== $password && !empty($password)) {
        $password = fast_store_sensitive_e($password, 'e');
    }
    return $password;
}
add_filter('fast_filter_field_sanitize_password', 'fast_sanitize_password', 10, 1);
