<?php
/**
 * Admin only actions
 *
 * @package Fast Plugin
 *
 * @copyright 2017 Oxygenna.com
 * @license http://themeforest.net/licenses/standard
 * @version 1.5.0
 * @author Oxygenna
 */

/*
* fast_filter_ticket_roles
*/

/**
 * Returns the plugin roles.
 *
 * @param array $roles Array of roles.
 */
function fast_get_ticket_roles($roles = array())
{
    return array_merge($roles, array(
        'fast_customer' => esc_html__('Ticket Customer', 'fast-plugin'),
        'fast_agent'    => esc_html__('Ticket Agent', 'fast-plugin'),
        'fast_manager'  => esc_html__('Ticket Manager', 'fast-plugin')
    ));
}
add_filter('fast_filter_ticket_roles', 'fast_get_ticket_roles', 10, 1);

/*
* fast_filter_ticket_capabilities
*/

/**
 * Returns the plugin capabilities.
 *
 * @param array $capabilities Array of capabilities.
 */
function fast_get_ticket_capabilities($capabilities = array())
{
    return array_merge($capabilities, array(
        'read'                  => esc_html__('Read', 'fast-plugin'),
        'edit_ticket'           => esc_html__('Edit Ticket', 'fast-plugin'),
        'read_ticket'           => esc_html__('Read Ticket', 'fast-plugin'),
        'delete_ticket'         => esc_html__('Delete Ticket', 'fast-plugin'),
        'edit_tickets'          => esc_html__('Edit Tickets', 'fast-plugin'),
        'edit_others_tickets'   => esc_html__('Edit Others Tickets', 'fast-plugin'),
        'publish_tickets'       => esc_html__('Publish Tickets', 'fast-plugin'),
        'read_private_tickets'  => esc_html__('Read Private Tickets', 'fast-plugin'),
        'manage_statuses'       => esc_html__('Manage Statuses', 'fast-plugin'),
        'assign_status'         => esc_html__('Change Ticket Status', 'fast-plugin'),
        'manage_agents'         => esc_html__('Manage Agents', 'fast-plugin'),
        'assign_agent'          => esc_html__('Change Ticket Agent', 'fast-plugin'),
        'manage_products'       => esc_html__('Manage Products', 'fast-plugin'),
        'assign_product'        => esc_html__('Change Ticket Product', 'fast-plugin'),
        'upload_files'          => esc_html__('Upload Images', 'fast-plugin'),
        'delete_posts'          => esc_html__('Delete Attachments', 'fast-plugin'),
        'edit_reply'            => esc_html__('Edit Reply', 'fast-plugin'),
        'edit_others_replies'   => esc_html__('Edit Others Replies', 'fast-plugin'),
        'read_reply'            => esc_html__('Read Reply', 'fast-plugin'),
        'delete_reply'          => esc_html__('Delete Reply', 'fast-plugin'),
        'edit_replies'          => esc_html__('Edit Replies', 'fast-plugin'),
        'publish_replies'       => esc_html__('Publish Replies', 'fast-plugin'),
        'delete_reply'          => esc_html__('Delete Reply', 'fast-plugin'),
        'edit_purchase'         => esc_html__('Edit Purchase', 'fast-plugin'),
        'edit_others_purchases' => esc_html__('Edit Others Purchases', 'fast-plugin'),
        'read_purchase'         => esc_html__('Read Purchase', 'fast-plugin'),
        'edit_purchase'         => esc_html__('Edit Purchase', 'fast-plugin'),
        'delete_purchase'       => esc_html__('Delete Purchase', 'fast-plugin'),
        'edit_purchases'        => esc_html__('Edit Purchases', 'fast-plugin'),
        'publish_purchases'     => esc_html__('Publish Purchases', 'fast-plugin'),
        'delete_purchase'       => esc_html__('Delete Purchase', 'fast-plugin')
    ));
}
add_filter('fast_filter_ticket_capabilities', 'fast_get_ticket_capabilities', 10, 1);

/*
* fast_filter_default_fast_customer_capabilities
*/

/**
 * Returns the default customer capabilities.
 *
 * @param array $capabilities Array of capabilities.
 */
function fast_get_customer_capabilities($capabilities = array())
{
    return array_merge($capabilities, array(
        'read',
        'read_ticket',
        'edit_ticket',
        'publish_tickets',
        'edit_tickets',
        'assign_product',
        'upload_files',
        'delete_posts',
        'read_purchase',
        'edit_purchase',
        'edit_purchases',
        'delete_purchase',
        'publish_purchases'
    ));
}
add_filter('fast_filter_default_fast_customer_capabilities', 'fast_get_customer_capabilities', 10, 1);

/*
* fast_filter_default_fast_agent_capabilities
*/

/**
 * Returns the default agent capabilities.
 *
 * @param array $capabilities Array of capabilities.
 */
function fast_get_agent_capabilities($capabilities = array())
{
    return array_merge($capabilities, array(
        'read',
        'edit_ticket',
        'read_ticket',
        'edit_tickets',
        'edit_others_tickets',
        'publish_tickets',
        'read_private_tickets',
        'assign_status',
        'assign_agent',
        'assign_product',
        'upload_files',
        'delete_posts',
        'edit_reply',
        'edit_others_replies',
        'read_reply',
        'delete_reply',
        'edit_replies',
        'publish_replies',
        'delete_reply',
        'read_purchase',
        'edit_purchase',
        'edit_purchases',
        'delete_purchase',
        'publish_purchases'
    ));
}
add_filter('fast_filter_default_fast_agent_capabilities', 'fast_get_agent_capabilities', 10, 1);

/*
* fast_filter_default_fast_manager_capabilities
*/

/**
 * Returns the default manager capabilities.
 *
 * @param array $capabilities Array of capabilities.
 */
function fast_get_manager_capabilities($capabilities = array())
{
    return array_merge($capabilities, array(
        'read',
        'edit_ticket',
        'read_ticket',
        'delete_ticket',
        'edit_tickets',
        'edit_others_tickets',
        'publish_tickets',
        'read_private_tickets',
        'manage_statuses',
        'assign_status',
        'manage_agents',
        'assign_agent',
        'manage_products',
        'assign_product',
        'upload_files',
        'delete_posts',
        'edit_reply',
        'edit_others_replies',
        'read_reply',
        'delete_reply',
        'edit_replies',
        'publish_replies',
        'delete_reply',
        'read_purchase',
        'edit_purchase',
        'edit_purchases',
        'delete_purchase',
        'publish_purchases'
    ));
}
add_filter('fast_filter_default_fast_manager_capabilities', 'fast_get_manager_capabilities', 10, 1);

/*
* fast_filter_field_sanitize_html
*/

/**
 * Makes sure that only allowed html entities are stored in options
 *
 * @param array $html Options framework option value.
 */
function fast_field_sanitize_html($html)
{
    return wp_kses($html, fast_get_allowed_html_tags());
}
add_filter('fast_filter_field_sanitize_html', 'fast_field_sanitize_html');

/*
* fast_filter_mail_replace_shortcodes
*/

/**
 * Replaces shortcodes in email templates with their respective values
 *
 * @param string $content mail markup.
 * @param array  $shortcodes mail shortcodes.
 */
function fast_mail_replace_shortcodes($content, $shortcodes)
{
    foreach ($shortcodes as $sc => $value) {
        $content = str_replace($sc, $value, $content);
    }
    return $content;
}
add_filter('fast_filter_mail_replace_shortcodes', 'fast_mail_replace_shortcodes', 10, 2);

/*
* fast_filter_build_placeholders
*/

/**
 * Replaces shortcodes in email templates with their respective values
 *
 * @param string $ticket   Ticket data.
 * @param object $comment  Comment object.
 * @param object $sender   Sender user.
 * @param object $receiver Receiver user.
 */
function fast_content_build_placeholders($ticket, $comment, $sender, $receiver)
{
    $ticket_author_user = get_userdata($ticket->post_author);

    $placeholders = array(
        '[ticket_id]'        => '#' . $ticket->ID,
        '[sender_name]'      => $sender->display_name,
        '[ticket_title]'     => $ticket->post_title,
        '[ticket_author]'    => $ticket_author_user->display_name,
        '[comment_content]'  => false === $comment ? '' : $comment->comment_content,
        '[slack_link]'       => sprintf(
            '<%1$s|' . esc_html__('Ticket Link', 'fast-plugin') . '>',
            esc_url(get_permalink($ticket->ID))
        )
    );

    // Add product name if we have one.
    $product = get_the_terms($ticket->ID, 'fast_product');
    if (!empty($product)) {
        $placeholders['[product]'] = $product[0]->name;
    }

    // Add Order number if we have one.
    $order_id = get_post_meta($ticket->ID, '_woocommerce_order_id', true);
    if (fast_is_woocommerce_active() && !empty($order_id)) {
        $placeholders['[order_number]'] = $order_id;
    }

    return $placeholders;
}
add_filter('fast_filter_build_placeholders', 'fast_content_build_placeholders', 10, 4);

/**
 * Replaces shortcodes in email templates with their respective values
 *
 * @param string $content Content to change.
 * @param array  $ticket Ticket data.
 * @param object $comment Comment object.
 * @param object $sender   Sender user.
 * @param object $receiver Receiver user.
 */
function fast_content_replace_placeholders($content, $ticket, $comment, $sender, $receiver)
{
    $placeholders = apply_filters('fast_filter_build_placeholders', $ticket, $comment, $sender, $receiver);
    foreach ($placeholders as $placeholder => $value) {
        $content = str_replace($placeholder, $value, $content);
    }
    return $content;
}
add_filter('fast_filter_replace_placeholders', 'fast_content_replace_placeholders', 10, 5);


/*
* rest_fast_ticket_query
*/

/**
 * Add the filter parameter
 *
 * @param  array           $args    The query arguments.
 * @param  WP_REST_Request $request Full details about the request.
 * @return array $args.
 **/
function fast_rest_api_filter_add_filter_param($args, $request)
{
    $user_id = get_current_user_id();
    $is_agent = fast_is_user_agent($user_id);
    $filters = $is_agent ? apply_filters('fast_filter_get_agent_filters', $user_id) : apply_filters('fast_filter_get_client_filters', $user_id);

    $request_filter = $request->get_param('filter');
    foreach ($filters as $key => $filter) {
        if ($request_filter === $filter['id']) {
            $active_filter = $filter;
        }
    }
    if (!empty($active_filter)) {
        $args = array_merge($args, $active_filter['args']);
        $pages = $request->get_param('pages');
        $args['posts_per_page'] *= empty($pages) ? 1 : $pages;
    }
    return $args;
}
/* Added in actions.php. */

/*
* fast_filter_get_agent_filters
*/

/**
 * Returns the agent filters
 *
 * @param string $user_id id of the current user.
 * @return array $filters.
 **/
function fast_get_agent_filters($user_id)
{
    $agent_id = fast_get_user_agent_id($user_id);
    return apply_filters('fast_filter_modify_agent_filters', array_merge(
        array(
            array(
                'id' => 'my-tickets',
                'name' => esc_html__('My Tickets', 'fast-plugin'),
                'args' => array(
                    'posts_per_page' => 25,
                    'post_status'    => 'publish,private',
                    'tax_query'      => array(
                        array(
                            'taxonomy' => 'fast_agent',
                            'field'    => 'term_id',
                            'terms'    => $agent_id,
                        ),
                        array(
                            'taxonomy' => 'fast_status',
                            'field'    => 'slug',
                            'terms'    => 'open'
                        ),
                    ),
                )
            )
        ),
        fast_create_filters_from_taxonomy_terms(array('fast_agent', 'fast_product', 'fast_status'))
    ), $user_id);
}
add_filter('fast_filter_get_agent_filters', 'fast_get_agent_filters', 10, 2);

/*
* fast_filter_modify_agent_filters
*/

/**
 * Adds ordering to all agent filters - order by agent replied and date
 *
 * @param array $filters All agent filters.
 * @return array Modified agent filters.
 */
function fast_add_ordering_to_agent_filters($filters)
{
    foreach ($filters as &$filter) {
        $filter['args']['orderby'] = array(
            'meta_value' => 'ASC',
            'modified'   => 'ASC'
        );
        $filter['args']['meta_key'] = '_agent_replied';
    }
    return $filters;
}
add_filter('fast_filter_modify_agent_filters', 'fast_add_ordering_to_agent_filters', 10, 1);


/*
* fast_filter_get_client_filters
*/

/**
 * Returns the client filters
 *
 * @param string $user_id id of the current user.
 * @return array $filters.
 **/
function fast_get_client_filters($user_id)
{
    return apply_filters('fast_filter_modify_client_filters', array_merge(
        array(
            array(
                'id' => 'my-tickets',
                'name' => esc_html__('My Tickets', 'fast-plugin'),
                'args' => array(
                    'author' => $user_id,
                    'posts_per_page'=> 25,
                    'post_status' => 'publish,private'
                )
            )
        ),
        fast_create_filters_from_taxonomy_terms(array('fast_agent', 'fast_product', 'fast_status'))
    ), $user_id);
}
add_filter('fast_filter_get_client_filters', 'fast_get_client_filters', 10, 2);

/*
* fast_filter_modify_client_filters
*/

/**
 * Adds ordering to all agent filters - order by agent replied and date
 *
 * @param array $filters All agent filters.
 * @param int   $user_id current user id.
 * @return array Modified agent filters.
 */
function fast_add_ordering_to_customer_filters($filters, $user_id)
{
    foreach ($filters as &$filter) {
        $filter['args']['author'] = $user_id;
        $filter['args']['orderby'] = array(
            'meta_value' => 'DESC',
            'modified'   => 'DESC'
        );
        $filter['args']['meta_key'] = '_agent_replied';
    }
    return $filters;
}
add_filter('fast_filter_modify_client_filters', 'fast_add_ordering_to_customer_filters', 10, 2);

/*
* post_type_link
*/

/**
 * Changes ticket url to point to url in the app.
 *
 * @param object $permalink Post permalink object.
 * @param object $post Post object.
 */
function fast_change_ticket_view_link($permalink, $post)
{
    if ('fast_ticket' === $post->post_type) {
        $support_page_id = fast_get_option('support_page');
        if (!empty($support_page_id)) {
            $permalink = trailingslashit(get_the_permalink($support_page_id)) . 'ticket/' . $post->ID;
        }
    }
    return $permalink;
}
add_filter('post_type_link', 'fast_change_ticket_view_link', 10, 2);

/*
* the_title
*/

/**
 * Removes the Private: prefix from titles of tickets
 *
 * @param object $private_title_format Post title format.
 * @param object $id Post object.
 */
function fast_remove_private_from_ticket_title($private_title_format, $id)
{
    return get_post_type($id) === 'fast_ticket' ? '%s' : $private_title_format;
}
add_filter('private_title_format', 'fast_remove_private_from_ticket_title', 10, 2);

/*
* fast_filter_options
*/

/**
 * Plugin options
 *
 * @param array $options Plugin option pages.
 */
function fast_plugin_options($options)
{
    return array(
        array(
            'slug' => 'fast-plugin-general-options',
            'page' => array(
                'title' => esc_html__('Fast Support Ticket Plugin Options', 'fast-plugin'),
            ),
            'menu' => array(
                'title' => esc_html__('FAST Support', 'fast-plugin')
            ),
            'sections'   => array(
                'support-pages' => array(
                    'title'   => esc_html__('Support Page', 'fast-plugin'),
                    'header'  => esc_html__('This setting will define the page that is used to load the plugin support app.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'             => esc_html__('Support Page', 'fast-plugin'),
                            'desc'             => esc_html__('Set the page where your support system will be shown.', 'fast-plugin'),
                            'id'               => 'support_page',
                            'type'             => 'select',
                            'database'         => 'custom_post_type',
                            'custom_post_type' => 'page',
                            'default'          => '',
                            'empty_label'      => esc_html__('Not Set', 'fast-plugin'),
                        )
                    )
                ),
                'app-options' => array(
                    'title'   => esc_html__('App Options', 'fast-plugin'),
                    'header'  => esc_html__('Configure the features available in the app.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'       => esc_html__('Ticket Mode', 'fast-plugin'),
                            'desc'       => esc_html__('Choose the kind of ticket that your app will create.', 'fast-plugin'),
                            'id'         => 'new_ticket_type',
                            'type'    => 'select',
                            'options' => array(
                                'regular'     => esc_html__('Regular Mode', 'fast-plugin'),
                                'envato'      => esc_html__('Envato Mode', 'fast-plugin'),
                                'woocommerce' => esc_html__('WooCommerce Mode', 'fast-plugin'),
                            ),
                            'default' => 'regular',
                        ),
                        array(
                            'name'    => esc_html__('App Colors', 'fast-plugin'),
                            'desc'    => esc_html__('Set the color scheme for your app.', 'fast-plugin'),
                            'id'      => 'app_color',
                            'type'    => 'select',
                            'default' => 'cyan_yellow',
                            'options' => array(
                                'blue-grey_amber'         => esc_html('آبی خاکستری و کهربا', 'fast-plugin'),
                                'purple_orange'           => esc_html('بنفش و نارنجی', 'fast-plugin'),
                                'cyan_yellow'             => esc_html('فیروزه ای و زرد', 'fast-plugin'),
                                'red_cyan'                => esc_html('قرمز و فیروزه ای', 'fast-plugin'),
                                'brown_light-blue'        => esc_html('قهوه ای و آبی کمرنگ', 'fast-plugin'),
                                'lime_pink'               => esc_html('صورتی', 'fast-plugin'),
                                'deep-orange_teal'        => esc_html('نارنجی یا سرخ مایل به قرمز', 'fast-plugin'),
                                'light-blue_purple'       => esc_html('آبی روشن و بنفش', 'fast-plugin'),
                                'light-green_deep-orange' => esc_html('سبز روشن و نارنجی پررنگ', 'fast-plugin'),
                                'indigo_pink'             => esc_html('صورتی و نیلی', 'fast-plugin'),
                            ),
                        ),
                        array(
                            'name'    => esc_html__('رنگ هشدار فوآیکون', 'fast-plugin-admin'),
                            'id'      => 'favicon_color',
                            'desc'    => esc_html__('Sets the color of the favicon alert text.', 'fast-plugin-admin'),
                            'default' => '#FFFFFF',
                            'type'    => 'color',
                            'format'  => 'hex'
                        ),
                        array(
                            'name'    => esc_html__('رنگ پس زمینه هشدار فاوآیکون', 'fast-plugin-admin'),
                            'id'      => 'favicon_bg_color',
                            'desc'    => esc_html__('Sets the background color of the favicon alert.', 'fast-plugin-admin'),
                            'default' => '#FF0000',
                            'type'    => 'color',
                            'format'  => 'hex'
                        ),
                        array(
                            'name'    => esc_html__('Private Tickets', 'fast-plugin'),
                            'desc'    => esc_html__('Allows users to create private tickets.', 'fast-plugin'),
                            'id'      => 'private_tickets',
                            'type'    => 'select',
                            'default' => 'on',
                            'options' => array(
                                'on'     => esc_html__('User can choose Public or Private', 'fast-plugin'),
                                'off'    => esc_html__('Always use Public Tickets', 'fast-plugin'),
                                'always' => esc_html__('Always use Private Tickets', 'fast-plugin'),
                            ),
                        ),
                        array(
                            'name'    => esc_html__('Show Site URL Field', 'fast-plugin'),
                            'desc'    => esc_html__('Adds a site URL field to tickets.', 'fast-plugin'),
                            'id'      => 'show_site_url',
                            'type'    => 'select',
                            'default' => 'on',
                            'options' => array(
                                'on'  => esc_html__('Show', 'fast-plugin'),
                                'off' => esc_html__('Hide', 'fast-plugin'),
                            ),
                        ),
                        array(
                            'name'        => esc_html__('Fallback Ticket Agent', 'fast-plugin'),
                            'desc'        => esc_html__('Select a user that will be assigned tickets by default (if no agents are set in selected product).', 'fast-plugin'),
                            'id'          => 'default_agent',
                            'type'        => 'select',
                            'empty_label' => esc_html__('Not Set', 'fast-plugin'),
                            'filter'      => 'fast_filter_product_agent_ids_values',
                            'default'     => '',
                        ),
                        array(
                            'name'    => esc_html__('Agents Can Create Tickets', 'fast-plugin'),
                            'desc'    => esc_html__('Allow agents to create new tickets.', 'fast-plugin'),
                            'id'      => 'agent_ticket_creation',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'off',
                        ),
                        array(
                            'name'    => esc_html__('Comments Order', 'fast-plugin'),
                            'desc'    => esc_html__('Comments order in ticket view. Descending order renders the newest comments at the top.', 'fast-plugin'),
                            'id'      => 'ticket_comments_order',
                            'type'    => 'select',
                            'options' => array(
                                'asc'     => esc_html__('Ascending', 'fast-plugin'),
                                'desc'    => esc_html__('Descending', 'fast-plugin'),
                            ),
                            'default' => 'desc',
                        ),
                        array(
                            'name'    => esc_html__('حداقل فاصله پاسخ برای مشتریان', 'fast-plugin-admin'),
                            'id'      => 'ticket_minimum_reply_interval',
                            'desc'    => esc_html__('Sets the minimum time (in seconds) that should pass before a client can reply again.', 'fast-plugin-admin'),
                            'type'    => 'slider',
                            'default'   => 15,
                            'attr'      => array(
                                'max'       => 120,
                                'min'       => 1,
                                'step'      => 1
                            )
                        ),
                        array(
                            'name'    => esc_html__('Customers can reply to closed tickets', 'fast-plugin'),
                            'desc'    => esc_html__('Customers can reply to closed tickets and they will be reopened.', 'fast-plugin'),
                            'id'      => 'customer_reply_to_closed',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'on',
                        ),
                    )
                ),
                'google-options' => array(
                    'title'   => esc_html__('Google reCAPTCHA', 'fast-plugin'),
                    'header'  => esc_html__('Add a reCAPTCHA check to the user registration page.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'        => esc_html__('کلید سایت reCAPTCHA', 'fast-plugin'),
                            'desc'        => esc_html__('Add your site key from https://www.google.com/recaptcha/', 'fast-plugin'),
                            'id'          => 'recaptcha_site_key',
                            'type'        => 'text',
                            'default'     => ''
                        ),
                        array(
                            'name'        => esc_html__('کلید مخفی reCAPTCHA', 'fast-plugin'),
                            'desc'        => esc_html__('Add your secret key from https://www.google.com/recaptcha/', 'fast-plugin'),
                            'id'          => 'recaptcha_secret_key',
                            'type'        => 'text',
                            'default'     => ''
                        ),
                    )
                ),
            )
        ),
        array(
            'slug' => 'fast-plugin-registration-page',
            'page' => array(
                'title' => esc_html__('Registration Page Content', 'fast-plugin'),
            ),
            'menu' => array(
                'title' => esc_html__('Registration Page', 'fast-plugin')
            ),
            'sections'   => array(
                'register-page' => array(
                    'title'   => esc_html__('Registration Page', 'fast-plugin'),
                    'header'  => esc_html__('Set the content that is shown on the registration page.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Show Register Types', 'fast-plugin'),
                            'desc'    => esc_html__('Which register options to show on the register page.', 'fast-plugin'),
                            'id'      => 'register_types',
                            'select2' => true,
                            'type'    => 'select',
                            'options' => array(
                                'wordpress' => esc_html__('WordPress Register', 'fast-plugin'),
                                'envato'    => esc_html__('Envato Register', 'fast-plugin'),
                            ),
                            'default' => array(
                                'wordpress'
                            ),
                            'attributes' => array(
                                'class'    => 'fast-select2-option',
                                'multiple' => '',
                            )
                        ),
                        array(
                            'name'    => esc_html__('New User can choose language', 'fast-plugin'),
                            'desc'    => esc_html__('A select box will be available to allow the new user to run the app in their own language.', 'fast-plugin'),
                            'id'      => 'register_user_language',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'on',
                        ),
                        array(
                            'name'       => esc_html__('Register User Page Title', 'fast-plugin'),
                            'desc'       => esc_html__('Sets the title that is shown on the register user page.', 'fast-plugin'),
                            'id'         => 'register_page_title',
                            'type'       => 'text',
                            'default'    => esc_html__('Register for support', 'fast-plugin'),
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                        array(
                            'name'     => esc_html__('Register User Page Content', 'fast-plugin'),
                            'desc'     => esc_html__('Sets the content that is shown on the register user page.', 'fast-plugin'),
                            'id'       => 'register_page_content',
                            'type'     => 'editor',
                            'sanitize' => 'html',
                            'default'  => '<p>Are you having trouble?  Have no fear we are here to help you!</p><p>Just fill in your details to gain access to our support system and create a support ticket.</p><p>Our support team will then be in touch as soon as possible to help you.</p>'
                        ),
                        array(
                            'name'     => esc_html__('Register User Page Background Image', 'fast-plugin'),
                            'desc'     => esc_html__('Sets the background image that is shown on the register user page.', 'fast-plugin'),
                            'id'       => 'register_page_image',
                            'type'     => 'upload',
                            'store'    => 'url',
                            'default'  => ''
                        ),
                    )
                ),
            )
        ),
        array(
            'slug' => 'fast-plugin-login-page',
            'page' => array(
                'title' => esc_html__('Login Page Content', 'fast-plugin'),
            ),
            'menu' => array(
                'title' => esc_html__('Login Page', 'fast-plugin')
            ),
            'sections'   => array(
                'register-page' => array(
                    'title'   => esc_html__('Login Page', 'fast-plugin'),
                    'header'  => esc_html__('Set the content that is shown on the login page.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Show Login Types', 'fast-plugin'),
                            'desc'    => esc_html__('Which login options to show on the login page.', 'fast-plugin'),
                            'id'      => 'login_page_login_types',
                            'select2' => true,
                            'type'    => 'select',
                            'options' => array(
                                'wordpress' => esc_html__('WordPress Login', 'fast-plugin'),
                                'envato'  => esc_html__('Envato Login', 'fast-plugin'),
                            ),
                            'default' => array(
                                'wordpress'
                            ),
                            'attributes' => array(
                                'class'    => 'fast-select2-option',
                                'multiple' => '',
                            )
                        ),
                        array(
                            'name'       => esc_html__('Login Page Title', 'fast-plugin'),
                            'desc'       => esc_html__('Sets the title that is shown on the login page.', 'fast-plugin'),
                            'id'         => 'login_page_title',
                            'type'       => 'text',
                            'default'    => esc_html__('Support Login', 'fast-plugin'),
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                        array(
                            'name'     => esc_html__('Login Page Content', 'fast-plugin'),
                            'desc'     => esc_html__('Sets the content that is shown on the login page.', 'fast-plugin'),
                            'id'       => 'login_page_content',
                            'type'     => 'editor',
                            'sanitize' => 'html',
                            'default'  => '<p>Welcome to our support ticket system</p><p>Please login using the form</p>'
                        ),
                        array(
                            'name'     => esc_html__('Login Page Background Image', 'fast-plugin'),
                            'desc'     => esc_html__('Sets the background image that is shown on the login page.', 'fast-plugin'),
                            'id'       => 'login_page_image',
                            'type'     => 'upload',
                            'store'    => 'url',
                            'default'  => ''
                        ),
                    )
                ),
            )
        ),
        array(
            'slug' => 'fast-plugin-new-ticket-page',
            'page' => array(
                'title' => esc_html__('New Ticket Page', 'fast-plugin'),
            ),
            'menu' => array(
                'title' => esc_html__('New Ticket Page', 'fast-plugin')
            ),
            'sections'   => array(
                'register-page' => array(
                    'title'   => esc_html__('New Ticket Page', 'fast-plugin'),
                    'header'  => esc_html__('Set the content that is shown on the new ticket page.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'       => esc_html__('New Ticket Page Title', 'fast-plugin'),
                            'desc'       => esc_html__('Sets the title that is shown on the register user page.', 'fast-plugin'),
                            'id'         => 'new_ticket_page_title',
                            'type'       => 'text',
                            'default'    => esc_html__('Create a new ticket', 'fast-plugin'),
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                        array(
                            'name'     => esc_html__('New Ticket Page Content', 'fast-plugin'),
                            'desc'     => esc_html__('Sets the content that is shown on the new ticket page.', 'fast-plugin'),
                            'id'       => 'new_ticket_page_content',
                            'type'     => 'editor',
                            'sanitize' => 'html',
                            'default'  => '<p>Please fill in the form and describe as clearly as possible the problem you are having.</p><p>To help our support team please include a link to your website or a screenshot.</p>'
                        ),
                        array(
                            'name'     => esc_html__('New Ticket Page Background Image', 'fast-plugin'),
                            'desc'     => esc_html__('Sets the background image that is shown on the new ticket page.', 'fast-plugin'),
                            'id'       => 'new_ticket_page_image',
                            'type'     => 'upload',
                            'store'    => 'url',
                            'default'  => ''
                        ),
                    )
                ),
            )
        ),
        array(
            'slug' => 'fast-plugin-extra-fields',
            'page' => array(
                'title' => esc_html__('Ticket Extra Fields', 'fast-plugin'),
            ),
            'menu' => array(
                'title' => esc_html__('Extra Fields', 'fast-plugin')
            ),
            'hide_footer' => true,
            'sections'   => array()
        ),
        array(
            'slug' => 'fast-plugin-roles-capabilities',
            'page' => array(
                'title' => esc_html__('Roles & Capabilities', 'fast-plugin'),
            ),
            'menu' => array(
                'title' => esc_html__('User Permissions', 'fast-plugin')
            ),
            'sections'   => array(
                'capabilities' => array(
                    'title'   => esc_html__('Assign Capabilities to Roles', 'fast-plugin'),
                    'header'  => esc_html__('Sets the capabilities for all user roles.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'             => esc_html__('Customer Role', 'fast-plugin'),
                            'desc'             => esc_html__('Sets capabilites that a customer will have.', 'fast-plugin'),
                            'id'               => 'fast_customer_capabilities',
                            'select2'          => true,
                            'type'             => 'select',
                            'filter'           => 'fast_filter_ticket_capabilities',
                            'default'          => apply_filters('fast_filter_default_fast_customer_capabilities', array()),
                            'attributes'       => array(
                                'class' => 'fast-select2-option',
                                'multiple' => '',
                            )
                        ),
                        array(
                            'name'             => esc_html__('Agent Role', 'fast-plugin'),
                            'desc'             => esc_html__('Sets capabilites that an agent will have.', 'fast-plugin'),
                            'id'               => 'fast_agent_capabilities',
                            'type'             => 'select',
                            'select2'          => true,
                            'filter'           => 'fast_filter_ticket_capabilities',
                            'default'          => apply_filters('fast_filter_default_fast_agent_capabilities', array()),
                            'attributes'       => array(
                                'class' => 'fast-select2-option',
                                'multiple' => '',
                            )
                        ),
                        array(
                            'name'             => esc_html__('Manager Role', 'fast-plugin'),
                            'desc'             => esc_html__('Sets capabilites that a manager will have.', 'fast-plugin'),
                            'id'               => 'fast_manager_capabilities',
                            'type'             => 'select',
                            'select2'          => true,
                            'filter'           => 'fast_filter_ticket_capabilities',
                            'default'          => apply_filters('fast_filter_default_fast_manager_capabilities', array()),
                            'attributes'       => array(
                                'class' => 'fast-select2-option',
                                'multiple' => '',
                            )
                        )
                    )
                )
            )
        ),
        array(
            'slug' => 'fast-plugin-email-options',
            'page' => array(
                'title' => esc_html__('Email Options', 'fast-plugin'),
            ),
            'menu' => array(
                'title' => esc_html__('Email Options', 'fast-plugin')
            ),
            'sections'   => array(
                'ticket-email-general' => array(
                    'title'   => esc_html__('تنظیمات قالب ایمیل تیکت', 'fast-plugin'),
                    'header'  => '<p>اطلاعات اولیه ایمیل را بررسی و تنظیم کنید شما در قسمت ارسال ایمیل نیز میتوانید از کدهای کوتاه نیز استفاده کنمشاهده. See <a target="_blank" href="http://help.oxygenna.com/plugins/fast/documentation.html#customizing-emails">درباره افزونه بیشتر بدانید</a> اطلاعات درباره کد کوتاه.',
                    'fields' => array(
                        array(
                            'name'    => esc_html__('رنگ پس زمینه ایمیل', 'fast-plugin-admin'),
                            'id'      => 'ticket_email_template_bg_color',
                            'desc'    => esc_html__('Sets the background color of the email.', 'fast-plugin-admin'),
                            'default' => '#FBFAFA',
                            'type'    => 'color',
                            'format'  => 'hex'
                        ),
                        array(
                            'name'     => esc_html__('تصویر لوگوی هدر', 'fast-plugin-admin'),
                            'desc'     => esc_html__('Sets the logo image that is shown in the header at the top of them email.', 'fast-plugin-admin'),
                            'id'       => 'ticket_email_logo_image',
                            'type'     => 'upload',
                            'store'    => 'url',
                            'default'  => ''
                        ),
                        array(
                            'name'    => esc_html__('رنگ پس زمینه لوگو هدر', 'fast-plugin-admin'),
                            'id'      => 'ticket_email_logo_image_bg_color',
                            'desc'    => esc_html__('Sets the background color of the email header.', 'fast-plugin-admin'),
                            'default' => '#FFFFFF',
                            'type'    => 'color',
                            'format'  => 'hex'
                        ),
                        array(
                            'name'     => esc_html__('متن هدر ایمیل', 'fast-plugin-admin'),
                            'desc'     => esc_html__('Sets the text that is shown at the top of the email below the logo.', 'fast-plugin-admin'),
                            'id'       => 'ticket_email_logo_title',
                            'type'     => 'text',
                            'default'  => 'FAST Support Ticket Plugin',
                            'sanitize' => 'html',
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                        array(
                            'name'    => esc_html__('رنگ متن هدر', 'fast-plugin-admin'),
                            'id'      => 'ticket_email_logo_title_color',
                            'desc'    => esc_html__('Sets the text color of the title area.', 'fast-plugin-admin'),
                            'default' => '#555555',
                            'type'    => 'color',
                            'format'  => 'hex'
                        ),
                        array(
                            'name'    => esc_html__('رنگ پس زمینه هدر', 'fast-plugin-admin'),
                            'id'      => 'ticket_email_logo_title_bg_color',
                            'desc'    => esc_html__('Sets the background color of the text area.', 'fast-plugin-admin'),
                            'default' => '#FFFFFF',
                            'type'    => 'color',
                            'format'  => 'hex'
                        ),
                        array(
                            'name'    => esc_html__('رنگ تقسیم کننده', 'fast-plugin-admin'),
                            'id'      => 'ticket_email_divider_color',
                            'desc'    => esc_html__('Sets the color of the divider that separates the title from the email content. The divider will appear only if there is a logo image or header text set.', 'fast-plugin-admin'),
                            'default' => '#FBFAFA',
                            'type'    => 'color',
                            'format'  => 'hex'
                        ),
                        array(
                            'name'    => esc_html__('شعاع مرزی', 'fast-plugin-admin'),
                            'id'      => 'ticket_email_border_radius',
                            'desc'    => esc_html__('Sets the border radius on the content area of the email.', 'fast-plugin-admin'),
                            'type'    => 'slider',
                            'default'   => 4,
                            'attr'      => array(
                                'max'       => 20,
                                'min'       => 0,
                                'step'      => 1
                            )
                        ),
                        array(
                            'name'    => esc_html__('رنگ متن دکمه', 'fast-plugin-admin'),
                            'id'      => 'ticket_email_button_color',
                            'desc'    => esc_html__('Sets the text color of the button.', 'fast-plugin-admin'),
                            'default' => '#FFFFFF',
                            'type'    => 'color',
                            'format'  => 'hex'
                        ),
                        array(
                            'name'    => esc_html__('رنگ پس زمینه دکمه', 'fast-plugin-admin'),
                            'id'      => 'ticket_email_button_bg_color',
                            'desc'    => esc_html__('Sets the background color of the button.', 'fast-plugin-admin'),
                            'default' => '#E86468',
                            'type'    => 'color',
                            'format'  => 'hex'
                        ),
                        array(
                            'name'        => esc_html__('متن دکمه بالای تیکن', 'fast-plugin-admin'),
                            'desc'        => esc_html__('Set the text that shows before the ticket button.', 'fast-plugin-admin'),
                            'id'          => 'ticket_above_ticket_button',
                            'type'        => 'text',
                            'default'     => '',
                            'sanitize'    => 'html',
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                        array(
                            'name'        => esc_html__('متن فوتر قالب ایمیل', 'fast-plugin-admin'),
                            'desc'        => esc_html__('Set the footer text that is shown inthe email template.', 'fast-plugin-admin'),
                            'id'          => 'ticket_email_footer',
                            'type'        => 'text',
                            'default'     => 'Thanks for using our support.',
                            'sanitize'    => 'html',
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                        array(
                            'name'    => esc_html__('رنگ متن فوتر', 'fast-plugin-admin'),
                            'id'      => 'ticket_email_footer_color',
                            'desc'    => esc_html__('Sets the text color of the footer.', 'fast-plugin-admin'),
                            'default' => '#E0A8AA',
                            'type'    => 'color',
                            'format'  => 'hex'
                        ),
                    )
                ),
                'ticket-new-email' => array(
                    'title'   => esc_html__('ایمیل تیکت جدید', 'fast-plugin'),
                    'header'  => esc_html__('Set the content of a new ticket email.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Send Email on New ticket', 'fast-plugin'),
                            'desc'    => esc_html__('Send an email to the assigned agent upon ticket creation', 'fast-plugin'),
                            'id'      => 'email_new_ticket',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'on',
                        ),
                        array(
                            'name'        => esc_html__('New Ticket Subject', 'fast-plugin'),
                            'desc'        => esc_html__('Sets the subject of the new email for the agent.', 'fast-plugin'),
                            'id'          => 'ticket_new_email_subject',
                            'type'        => 'text',
                            'default'     => 'New ticket [ticket_id] from [sender_name] for [product]',
                            'sanitize'    => 'html',
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                        array(
                            'name'        => esc_html__('New Ticket Content', 'fast-plugin'),
                            'desc'        => esc_html__('Set the content for the new email for the agent.', 'fast-plugin'),
                            'id'          => 'ticket_new_email_content',
                            'type'        => 'editor',
                            'default'     => '<p><strong>You have a new support ticket from [sender_name]</strong></p><p>Here is what they said:</p>[comment_content]',
                            'sanitize'    => 'html'
                        ),
                    )
                ),
                'ticket-reply-email' => array(
                    'title'   => esc_html__('Ticket Reply Email', 'fast-plugin'),
                    'header'  => esc_html__('Set the content of a ticket reply email.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Send Email on Client Reply', 'fast-plugin'),
                            'desc'    => esc_html__('Send an Email to agent upon client\'s reply.', 'fast-plugin'),
                            'id'      => 'email_client_reply',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'on',
                        ),
                        array(
                            'name'    => esc_html__('Send Email on Agent Reply', 'fast-plugin'),
                            'desc'    => esc_html__('Send an Email to client upon agent\'s reply.', 'fast-plugin'),
                            'id'      => 'email_agent_reply',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'on',
                        ),
                        array(
                            'name'        => esc_html__('Ticket Reply Subject', 'fast-plugin'),
                            'desc'        => esc_html__('Set the reply email subject.', 'fast-plugin'),
                            'id'          => 'ticket_reply_email_subject',
                            'type'        => 'text',
                            'default'     => 'Comment from [sender_name] on ticket [ticket_id]',
                            'sanitize'    => 'html',
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                        array(
                            'name'        => esc_html__('Ticket Reply Content', 'fast-plugin'),
                            'desc'        => esc_html__('Sets the email content for reply email.', 'fast-plugin'),
                            'id'          => 'ticket_reply_email_content',
                            'type'        => 'editor',
                            'default'     => '<p><strong>There is a new comment from [sender_name] in the support ticket [ticket_title]</strong></p>[comment_content]',
                            'sanitize'    => 'html'
                        ),
                    )
                ),
                'ticket-assign-email' => array(
                    'title'   => esc_html__('Ticket Assign Email', 'fast-plugin'),
                    'header'  => esc_html__('Set the content of a ticket assign email.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Send Email on Ticket Assignment', 'fast-plugin'),
                            'desc'    => esc_html__('Send an email upon ticket assignment.', 'fast-plugin'),
                            'id'      => 'email_ticket_assignment',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'off',
                        ),
                        array(
                            'name'        => esc_html__('Ticket Assign Subject', 'fast-plugin'),
                            'desc'        => esc_html__('Sets the subject of the mail that is sent when a ticket is assigned to an agent', 'fast-plugin'),
                            'id'          => 'ticket_assign_email_subject',
                            'type'        => 'text',
                            'default'     => '[sender_name] assigned you ticket [ticket_id]',
                            'sanitize'    => 'html',
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                        array(
                            'name'        => esc_html__('Ticket Assign Content', 'fast-plugin'),
                            'desc'        => esc_html__('Set the email content for the ticket assigned email for the agent.', 'fast-plugin'),
                            'id'          => 'ticket_assign_email_content',
                            'type'        => 'editor',
                            'default'     => '<p>[sender_name] just assigned you the ticket [ticket_title] from [ticket_author]</p>',
                            'sanitize'    => 'html'
                        ),
                    )
                ),
                'user-new-email' => array(
                    'title'   => esc_html__('New User Email', 'fast-plugin'),
                    'header'  => esc_html__('Set the content of a new user email.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Send Email on User Registration', 'fast-plugin'),
                            'desc'    => esc_html__('Send an email to the a user when they register or send an email to the system.', 'fast-plugin'),
                            'id'      => 'email_new_user',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'on',
                        ),
                        array(
                            'name'        => esc_html__('New User Subject', 'fast-plugin'),
                            'desc'        => esc_html__('Sets the subject of the new user email.', 'fast-plugin'),
                            'id'          => 'user_new_email_subject',
                            'type'        => 'text',
                            'default'     => 'Thank you for registering with our support.',
                            'sanitize'    => 'html',
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                        array(
                            'name'        => esc_html__('New User Content', 'fast-plugin'),
                            'desc'        => esc_html__('Set the content for the new user email.', 'fast-plugin'),
                            'id'          => 'user_new_email_content',
                            'type'        => 'editor',
                            'default'     => '<p><strong>Welcome to our support.</strong></p><p>Here are your new user details:</p>',
                            'sanitize'    => 'html'
                        ),
                    )
                ),
            )
        ),
        array(
            'slug' => 'fast-plugin-slack',
            'page' => array(
                'title' => esc_html__('Slack Integration Options', 'fast-plugin'),
            ),
            'menu' => array(
                'title' => esc_html__('Slack Integration', 'fast-plugin')
            ),
            'sections'   => array(
                'slack-integration' => array(
                    'title'   => esc_html__('Slack Integration', 'fast-plugin'),
                    'header'  => '<p>تنظیمات اولیه در مورد اطلاع رسانی کاربران.</p><p>دستورالعمل کامل <a target="_blank" href="http://help.oxygenna.com/plugins/fast/documentation.html#slack-intergration">توضیحات بیشتر</a></p>',
                    'fields' => array(
                        array(
                            'name'       => esc_html__('Webhook URL', 'fast-plugin'),
                            'desc'       => esc_html__('Enter the webhook URL of your Slack app in order to dispatch messages to Slack.', 'fast-plugin'),
                            'id'         => 'slack_webhook',
                            'type'       => 'text',
                            'default'    => '',
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                    )
                ),
                'ticket-new-slack' => array(
                    'title'   => esc_html__('New Ticket Notification', 'fast-plugin'),
                    'header'  => esc_html__('Set the content of a new ticket notification.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Notify on New ticket', 'fast-plugin'),
                            'desc'    => esc_html__('Dispatch a message to Slack upon ticket creation', 'fast-plugin'),
                            'id'      => 'slack_new_ticket',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'on',
                        ),
                        array(
                            'name'        => esc_html__('New Ticket Notification Content', 'fast-plugin'),
                            'desc'        => esc_html__('Set the content for the new ticket notification.', 'fast-plugin'),
                            'id'          => 'ticket_new_slack_content',
                            'type'        => 'text',
                            'default'     => 'There is a new support ticket for `[product]` from `[sender_name]`. Here is what they said: ```[comment_content]``` View the ticket here: [slack_link]',
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                    )
                ),
                'ticket-reply-slack' => array(
                    'title'   => esc_html__('Ticket Reply Notification', 'fast-plugin'),
                    'header'  => esc_html__('Set the content of a ticket reply notification.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Notify on Client Reply', 'fast-plugin'),
                            'desc'    => esc_html__('Dispatch a message to Slack upon client\'s reply.', 'fast-plugin'),
                            'id'      => 'slack_client_reply',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'on',
                        ),
                        array(
                            'name'    => esc_html__('Notify on Agent Reply', 'fast-plugin'),
                            'desc'    => esc_html__('Dispatch a message to Slack upon agent\'s reply.', 'fast-plugin'),
                            'id'      => 'slack_agent_reply',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'off',
                        ),
                        array(
                            'name'        => esc_html__('Ticket Reply Notification Content', 'fast-plugin'),
                            'desc'        => esc_html__('Sets the slack notification content for reply email.', 'fast-plugin'),
                            'id'          => 'ticket_reply_slack_content',
                            'type'        => 'text',
                            'default'     => 'There is a new comment for `[product]` from `[sender_name]` in the support ticket `[ticket_title]`. ```[comment_content]```  View the ticket here: [slack_link]',
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                    )
                ),
                'ticket-assign-slack' => array(
                    'title'   => esc_html__('Ticket Assign Notification', 'fast-plugin'),
                    'header'  => esc_html__('Set the content of a ticket assign notification.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Notify on Ticket Assignment', 'fast-plugin'),
                            'desc'    => esc_html__('Dispatch a message to Slack upon ticket assignment.', 'fast-plugin'),
                            'id'      => 'slack_ticket_assignment',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'off',
                        ),
                        array(
                            'name'        => esc_html__('Ticket Assign Content', 'fast-plugin'),
                            'desc'        => esc_html__('Set the content for the ticket assigned notification for the agent. [comment_content] and [comment_author] are not applicable here', 'fast-plugin'),
                            'id'          => 'ticket_assign_slack_content',
                            'type'        => 'text',
                            'default'     => '`[sender_name]` just reassigned the ticket `[ticket_title]` from `[ticket_author]`',
                            'attributes' => array(
                                'class'  => 'widefat'
                            )
                        ),
                    )
                )
            )
        ),
        array(
            'slug' => 'fast-plugin-woocommerce',
            'page' => array(
                'title' => esc_html__('WooCommerce Options', 'fast-plugin'),
            ),
            'menu' => array(
                'title' => esc_html__('WooCommerce', 'fast-plugin')
            ),
            'sections'   => array(
                'woocommerce-integration' => array(
                    'title'   => esc_html__('WooCommerce MyAccount Page', 'fast-plugin'),
                    'header'  => esc_html__('Sets the options for the WooCommerce Customer MyAccount page', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Add Support Link To My Account Menu', 'fast-plugin'),
                            'desc'    => esc_html__('Adds a link to the support tickets to customers MyAccount menu.', 'fast-plugin'),
                            'id'      => 'woocommerce_support_tickets_link',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'on',
                        ),
                        array(
                            'name'    => esc_html__('Add Support Button To My Orders Page', 'fast-plugin'),
                            'desc'    => esc_html__('Adds a link to the support tickets to customers MyAccount page.', 'fast-plugin'),
                            'id'      => 'woocommerce_support_tickets_order_button',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'on',
                        ),
                    )
                ),
                'woocommerce-agents' => array(
                    'title'   => esc_html__('WooCommerce Agents', 'fast-plugin'),
                    'header'  => esc_html__('Set the agents that will handle your WooCommerce support tickets. Each agent will be assigned a ticket in turn.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'             => esc_html__('WooCommerce Agents', 'fast-plugin'),
                            'id'               => 'woocommerce_agents',
                            'desc'             => esc_html__('Sets the fast agents that will handle all woocommerce support tickets.', 'fast-plugin'),
                            'type'             => 'select',
                            'filter'           => 'fast_filter_product_agent_ids_values',
                            'select2'          => true,
                            'default'          => array(),
                            'attributes'       => array(
                                'placeholder' => esc_html__('Select ticket agents who will handle woocommerce support tickets', 'fast-plugin'),
                                'class'       => 'fast-select2-option',
                                'multiple'    => '',
                            ),
                        )
                    )
                ),
            )
        ),
        array(
            'slug' => 'fast-plugin-email-pipe',
            'page' => array(
                'title' => esc_html__('Email Piping Options', 'fast-plugin'),
            ),
            'menu' => array(
                'title' => esc_html__('Email Piping', 'fast-plugin')
            ),
            'sections'   => array(
                'email-piping' => array(
                    'title'   => esc_html__('Email Piping', 'fast-plugin'),
                    'header'  => esc_html__('Setup FAST to allow use emails to respond / create tickets.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Enable Email Piping', 'fast-plugin'),
                            'desc'    => esc_html__('Turns on email piping.', 'fast-plugin'),
                            'id'      => 'emailp_enabled',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'off',
                        ),
                        array(
                            'name'    => esc_html__('Check Emails Every', 'fast-plugin'),
                            'desc'    => esc_html__('Set the time period to wait to check emails.', 'fast-plugin'),
                            'id'      => 'emailp_timespan',
                            'type'    => 'select',
                            'filter'  => 'fast_filter_cron_time_spans',
                            'default' => 'ten_minutes',
                        ),
                    )
                ),
                'email-piping-replies' => array(
                    'title'   => esc_html__('Reply to tickets', 'fast-plugin'),
                    'header'  => esc_html__('Setup FAST to allow users to reply to ticket emails.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Enable email replies to tickets', 'fast-plugin'),
                            'desc'    => esc_html__('Users can reply to emails to respond to tickets.', 'fast-plugin'),
                            'id'      => 'emailp_reply_enabled',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'off',
                        ),
                    )
                ),
                'email-piping-new' => array(
                    'title'   => esc_html__('Create tickets with email', 'fast-plugin'),
                    'header'  => esc_html__('Setup FAST to allow customers to create tickets with email.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Enable create ticket with email', 'fast-plugin'),
                            'desc'    => esc_html__('Users can create tickets using email.', 'fast-plugin'),
                            'id'      => 'emailp_create_enabled',
                            'type'    => 'select',
                            'options' => array(
                                'on'     => esc_html__('On', 'fast-plugin'),
                                'off'    => esc_html__('Off', 'fast-plugin'),
                            ),
                            'default' => 'off',
                        ),
                        array(
                            'name'    => esc_html__('Default New Ticket Product', 'fast-plugin'),
                            'desc'    => esc_html__('Select a product that will be assigned when user creates a ticket via email.', 'fast-plugin'),
                            'id'      => 'emailp_default_product',
                            'type'    => 'select',
                            'filter'  => 'fast_filter_product_product_ids_values',
                            'default' => '',
                        ),
                    )
                ),
                'email-piping-imap' => array(
                    'title'   => esc_html__('Email IMAP Details', 'fast-plugin'),
                    'header'  => esc_html__('Enter your IMAP server details so FAST can connect to your email', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'    => esc_html__('Email Address Username', 'fast-plugin'),
                            'desc'    => esc_html__('Email address username to login to email account.', 'fast-plugin'),
                            'id'      => 'emailp_username',
                            'type'    => 'text',
                            'default' => '',
                        ),
                        array(
                            'name'    => esc_html__('Email Address Password', 'fast-plugin'),
                            'desc'    => esc_html__('Email address password to login to email account.', 'fast-plugin'),
                            'id'      => 'emailp_password',
                            'type'    => 'text',
                            'default' => '',
                            'sanitize' => 'password',
                            'attributes'  => array(
                                'type' => 'password',
                            )
                        ),
                        array(
                            'name'    => esc_html__('IMAP Server URL', 'fast-plugin'),
                            'desc'    => esc_html__('Email server url address.', 'fast-plugin'),
                            'id'      => 'emailp_url',
                            'type'    => 'text',
                            'default' => '',
                        ),
                        array(
                            'name'    => esc_html__('IMAP Server port', 'fast-plugin'),
                            'desc'    => esc_html__('Port to use when connecting to email server.', 'fast-plugin'),
                            'id'      => 'emailp_port',
                            'type'    => 'text',
                            'default' => '',
                        ),
                        array(
                            'name'    => esc_html__('صندوق پستی IMAP', 'fast-plugin'),
                            'desc'    => esc_html__('Mailbox to check for emails (usually default INBOX is used).', 'fast-plugin'),
                            'id'      => 'emailp_mailbox',
                            'type'    => 'text',
                            'default' => 'INBOX',
                        ),
                        array(
                            'name'    => esc_html__('Encryption', 'fast-plugin'),
                            'desc'    => esc_html__('Set encryption to use when connecting to serve.', 'fast-plugin'),
                            'id'      => 'emailp_encryption',
                            'type'    => 'select',
                            'options' => array(
                                '/ssl'     => esc_html__('SSL', 'fast-plugin'),
                                '/tls'    => esc_html__('TLS', 'fast-plugin'),
                                ''    => esc_html__('None', 'fast-plugin'),
                            ),
                            'default' => '/ssl',
                        ),

                    )
                ),
                'email-piping-test' => array(
                    'title'   => esc_html__('Emil Piping Status', 'fast-plugin'),
                    'header'  => esc_html__('Check the status of your email piping.', 'fast-plugin'),
                    'fields' => array(
                        array(
                            'name'        => esc_html__('Last Email Check Results', 'fast-plugin'),
                            'desc'        => esc_html__('Shows the results of the last email run.', 'fast-plugin'),
                            'id'          => 'emailp_run_results',
                            'type'        => 'filter',
                            'filter'      => 'fast_filter_email_pipe_results'
                        ),
                        array(
                            'name'        => esc_html__('Next Email Check', 'fast-plugin'),
                            'desc'        => esc_html__('Shows the time the next email check will run.', 'fast-plugin'),
                            'id'          => 'emailp_run_time',
                            'type'        => 'filter',
                            'filter'      => 'fast_filter_email_pipe_next_run'
                        ),
                        array(
                            'name'        => esc_html__('Check Now', 'fast-plugin'),
                            'desc'        => esc_html__('Runs the email checker using the above settings.', 'fast-plugin'),
                            'id'          => 'emailp_run_check',
                            'type'        => 'button',
                            'button_text' => esc_html__('Run Email Check Now', 'fast-plugin'),
                            'attributes'  => array(
                                'class' => 'button',
                                'value' => 'run'
                            )
                        ),
                    )
                )
            )
        ),
        array(
            'slug' => 'fast-plugin-advanced',
            'page' => array(
                'title' => esc_html__('Advanced Options', 'fast-plugin'),
            ),
            'menu' => array(
                'title' => esc_html__('Advanced', 'fast-plugin')
            ),
            'sections'   => array(
                'css-options' => array(
                    'title'   => esc_html__('App CSS Options / Registration', 'fast-plugin'),
                    'header'  => '<p>کدهای سفارشی css</a></p>',
                    'fields' => array(
                        array(
                            'name'        => esc_html__('Extra CSS', 'fast-plugin'),
                            'desc'        => esc_html__('Adds extra CSS to the frontend app.', 'fast-plugin'),
                            'id'          => 'extra_css',
                            'type'        => 'textarea',
                            'default'     => '',
                            'attributes'  => array(
                                'class' => 'widefat',
                                'rows' => 10
                            )
                        ),
                    )
                )
            )
        )
    );
}
add_filter('fast_filter_options', 'fast_plugin_options');

/*
* fast_filter_metabox_options
*/

/**
 * Plugin metabox options
 *
 * @param array $options Plugin meabox options.
 */
function fast_plugin_metabox_options($options)
{
    return array(
        'product_fields' => array(
            'id' => 'ticket_product_fields',
            'taxonomies' => array(
                'fast_product'
            ),
            'fields' => array(
                array(
                    'name'             => esc_html__('Product Agents', 'fast-plugin'),
                    'id'               => 'product_user_ids',
                    'desc'             => esc_html__('Set a WordPress user(s) that will be assigned tickets for this product.', 'fast-plugin'),
                    'type'             => 'select',
                    'filter'           => 'fast_filter_product_agent_ids_values',
                    'select2'          => true,
                    'default'          => array(),
                    'attributes'       => array(
                        'placeholder' => esc_html__('Select ticket agents who will support this product', 'fast-plugin'),
                        'class'       => 'fast-select2-option',
                        'multiple'    => '',
                    ),
                )
            )
        ),
        'agent_fields' => array(
            'id' => 'ticket_agent_fields',
            'taxonomies' => array(
                'fast_agent'
            ),
            'fields' => array(
                array(
                    'name'             => esc_html__('WordPress User', 'fast-plugin'),
                    'id'               => 'agent_user_id',
                    'desc'             => esc_html__('Choose a WordPress user that corresponds to this agent.', 'fast-plugin'),
                    'type'             => 'select',
                    'empty_label'      => esc_html__('No User Set', 'fast-plugin'),
                    'filter'           => 'fast_filter_taxonomy_user_select_option_values',
                    'select2'          => true,
                    'default'          => 0,
                    'attributes'       => array(
                        'placeholder' => esc_html__('Select a WordPress user', 'fast-plugin'),
                        'class'       => 'fast-select2-option',
                    )
                )
            )
        ),
        'agent_visibility_fields' => array(
            'id' => 'ticket_agent_visibility_fields',
            'taxonomies' => array(
                'fast_agent',
            ),
            'fields' => array(
                array(
                    'name'             => esc_html__('Filter Visibility', 'fast-plugin'),
                    'id'               => 'fast_agent_role_visibility',
                    'desc'             => esc_html__('Choose the user roles that will be able to use this agent as a ticket filter.', 'fast-plugin'),
                    'type'             => 'select',
                    'empty_label'      => esc_html__('No Role Set', 'fast-plugin'),
                    'filter'           => 'fast_filter_ticket_roles',
                    'select2'          => true,
                    'default'          => array('fast_manager', 'fast_agent'),
                    'attributes'       => array(
                        'placeholder' => esc_html__('Select a user role', 'fast-plugin'),
                        'class'       => 'fast-select2-option',
                        'multiple'    => '',
                    )
                )
            )
        ),
        'product_visibility_fields' => array(
            'id' => 'ticket_product_visibility_fields',
            'taxonomies' => array(
                'fast_product',
            ),
            'fields' => array(
                array(
                    'name'             => esc_html__('Filter Visibility', 'fast-plugin'),
                    'id'               => 'fast_product_role_visibility',
                    'desc'             => esc_html__('Choose the user roles that will be able to use this product as a ticket filter.', 'fast-plugin'),
                    'type'             => 'select',
                    'empty_label'      => esc_html__('No Role Set', 'fast-plugin'),
                    'filter'           => 'fast_filter_ticket_roles',
                    'select2'          => true,
                    'default'          => array('fast_manager', 'fast_agent'),
                    'attributes'       => array(
                        'placeholder' => esc_html__('Select a user role', 'fast-plugin'),
                        'class'       => 'fast-select2-option',
                        'multiple'    => '',
                    )
                )
            )
        ),
        'status_visibility_fields' => array(
            'id' => 'ticket_status_visibility_fieldz',
            'taxonomies' => array(
                'fast_status'
            ),
            'fields' => array(
                array(
                    'name'             => esc_html__('Filter Visibility', 'fast-plugin'),
                    'id'               => 'status_role_visibility',
                    'desc'             => esc_html__('Choose the user roles that will be able to use this status as a ticket filter.', 'fast-plugin'),
                    'type'             => 'select',
                    'empty_label'      => esc_html__('No Role Set', 'fast-plugin'),
                    'filter'           => 'fast_filter_ticket_roles',
                    'select2'          => true,
                    'default'          => array('fast_manager', 'fast_agent', 'fast_customer'),
                    'attributes'       => array(
                        'placeholder' => esc_html__('Select a user role', 'fast-plugin'),
                        'class'       => 'fast-select2-option',
                        'multiple'    => '',
                    )
                )
            )
        ),
    );
}
add_filter('fast_filter_metabox_options', 'fast_plugin_metabox_options', 1, 1);

/*
* duplicate_comment_id
*/

/**
 * Allow duplicate ticket comments.
 */
add_filter('duplicate_comment_id', '__return_false');

/*
* rest_post_dispatch
*/

/**
 * Add custom header in api response.
 *
 * @param object  $response Response Object.
 * @param boolean $server Server Instance.
 * @param object  $request Request Object.
 */
function fast_rest_api_custom_header($response, $server, $request)
{
    if ('GET' === $request->get_method() && '/wp/v2/tickets' === $request->get_route()) {
        $response->header('Access-Control-Expose-Headers', 'X-WP-Total, X-WP-TotalPages, ticket-filters');
    }
    return $response;
}
add_filter('rest_post_dispatch', 'fast_rest_api_custom_header', 10, 3);

/*
* rest_pre_insert_comment
*/

/**
 * Add custom header in api response.
 *
 * @param array  $prepared_comment Comment.
 * @param object $request HTTP Request.
 */
function fast_rest_remove_html_filter_for_ticket_comments($prepared_comment, $request)
{
    if ('fast_ticket' === get_post_type($prepared_comment['comment_post_ID'])) {
        remove_filter('pre_comment_content', 'wp_filter_kses');
    }
    return $prepared_comment;
}
add_filter('rest_pre_insert_comment', 'fast_rest_remove_html_filter_for_ticket_comments', 10, 2);

/**
 * Override default discussion options, always allow comments in tickets.
 *
 * @param array $data Post Data.
 */
function fast_always_allow_ticket_comments($data)
{
    if ('fast_ticket' === $data['post_type']) {
        $data['comment_status'] = 'open';
    }
    return $data;
}
add_filter('wp_insert_post_data', 'fast_always_allow_ticket_comments');

/*
* map_meta_cap
*/

/**
 * Maps capabilities for ticket posts
 *
 * @param array  $caps Returns the user's actual capabilities.
 * @param string $cap Capability name.
 * @param int    $user_id The user ID.
 * @param array  $args Adds the context to the cap. Typically the object ID.
 *
 * @return array Capabilities to check.
 */
function fast_ticket_map_meta_cap($caps, $cap, $user_id, $args)
{
    /* If editing, deleting, or reading a ticket, get the post and post type object. */
    if ('edit_ticket' === $cap || 'delete_ticket' === $cap || 'read_ticket' === $cap) {
        if (empty($args)) {
            return array();
        }

        $post = get_post($args[0]);
        $post_type = get_post_type_object($post->post_type);

        /* Set an empty array for the caps. */
        $caps = array();
    }

    /* If editing a ticket, assign the required capability. */
    switch ($cap) {
        case 'edit_ticket':
            if ($user_id == $post->post_author) {
                $caps[] = $post_type->cap->edit_posts;
            } else {
                $caps[] = $post_type->cap->edit_others_posts;
            }
            break;
        case 'delete_ticket':
            if ($user_id == $post->post_author) {
                $caps[] = $post_type->cap->delete_posts;
            } else {
                $caps[] = $post_type->cap->delete_others_posts;
            }
            break;
        case 'read_ticket':
            if ('private' === $post->post_status) {
                if ($user_id == $post->post_author) {
                    $caps[] = $post_type->cap->edit_posts;
                } else {
                    $caps[] = $post_type->cap->read_private_posts;
                }
            } else {
                if ($user_id == $post->post_author) {
                    $caps[] = $post_type->cap->edit_posts;
                } else {
                    $caps[] = $post_type->cap->edit_others_posts;
                }
            }
            break;
    }

    /* Return the capabilities required by the user. */
    return $caps;
}
add_filter('map_meta_cap', 'fast_ticket_map_meta_cap', 10, 4);

/*
* pre_comment_approved
*/

/**
 * Filters a comment's approval status before it is set. We always approve ticket comments
 *
 * @param bool|string $approved     The approval status. Accepts 1, 0, or 'spam'.
 * @param array       $comment_data Comment data.
 */
function fast_always_approve_ticket_comment($approved, $comment_data)
{
    if ('fast_ticket' === get_post_type($comment_data['comment_post_ID'])) {
        $approved = 1;
    }
    return $approved;
}
add_filter('pre_comment_approved', 'fast_always_approve_ticket_comment', 99, 2);

/*
* get_avatar_url
*/

/**
 * Use custom avatars in case WP User Avatar plugin is installed.
 *
 * @param string $url           avatar url.
 * @param string $id_or_email   user id or email.
 * @param object $args          avatar args.
 */
function fast_custom_avatar_url($url, $id_or_email, $args)
{
    global $wpua_functions;
    if (null !== $wpua_functions) {
        remove_filter('get_avatar_url', 'fast_custom_avatar_url', 10);
        // Check if this user has an custom avatar.
        if (has_wp_user_avatar($id_or_email)) {
            // Return custom avatar set by user.
            $url = get_wp_user_avatar_src($id_or_email, $args['size']);
        } elseif (!$wpua_functions->wpua_has_gravatar($id_or_email)) {
            // Get default avatar set by plugin.
            $default_image_details = $wpua_functions->wpua_default_image($args['size']);
            $url = $default_image_details['src'];
        }
        add_filter('get_avatar_url', 'fast_custom_avatar_url', 10, 3);
    }
    return $url;
}
add_filter('get_avatar_url', 'fast_custom_avatar_url', 10, 3);

/*
* rest_fast_ticket_query
*/

/**
 * Adds the needs_reply=0|1 to the ticket api.
 *
 * @param array  $args Query vars to use.
 * @param object $request API request.
 *
 * @return array New query args.
 */
function fast_add_needs_reply_var_to_ticket_api($args, $request)
{
    $agent_replied = $request->get_param('agent_replied');
    if (null !== $agent_replied) {
        // Create meta_query if not already there.
        if (!isset($args['meta_query'])) {
            $args['meta_query'] = array();
        }
        // Add query for agent replied.
        array_push($args['meta_query'], array(
            'key'     => '_agent_replied',
            'value'   => $agent_replied,
        ));
    }
    return $args;
}
add_filter('rest_fast_ticket_query', 'fast_add_needs_reply_var_to_ticket_api', 10, 2);

/*
* redirect_canonical
*/

/**
 * Disables redirect to / when frontpage set in General -> Reading.
 * used to allow extra endpoints like /ticket /login to work when
 * support page is set as the frontpage.
 *
 * @param boolean $redirect To redirect or not.
 **/
function fast_disable_canonical_redirect_for_front_page($redirect)
{
    if (is_page()) {
        $front_page = get_option('page_on_front');
        if (is_page($front_page)) {
            $redirect = false;
        }
    }
    return $redirect;
}
/* added by fast_rewrite_app_endpoints in actions.php */

/*
 * widget_comments_args
 */

/**
 * Adds custom arg in widget comment query
 *
 * @param  array $comment_args Comment Query Args.
 * @return array Modified Query Args.
 */
function fast_custom_comments_args_for_widgets($comment_args)
{
    $comment_args['post_type_not_in'] = 'fast_ticket';
    return $comment_args;
}
add_filter('widget_comments_args', 'fast_custom_comments_args_for_widgets');

/*
 * comments_clauses
 */

/**
 * Exclude comments of the fast_ticket post type in widgets.
 *
 * @param  array  $clauses Query Clauses.
 * @param  object $wp_comment_query Actual Comment Query.
 * @return array Modified Query Clauses.
 */
function fast_remove_ticket_comments_from_widgets($clauses, $wp_comment_query)
{
    global $wpdb;
    if (isset($wp_comment_query->query_vars['post_type_not_in'])) {
        if (!$clauses['join']) {
            $clauses['join'] = 'JOIN $wpdb->posts ON $wpdb->posts.ID = $wpdb->comments.comment_post_ID';
        }

        if (!$wp_comment_query->query_vars['post_type']) {
            $clauses['where'] .= $wpdb->prepare(" AND {$wpdb->posts}.post_type != %s", 'fast_ticket');
        }
        unset($wp_comment_query->query_vars['post_type_not_in']);
    }
    return $clauses;
}
add_filter('comments_clauses', 'fast_remove_ticket_comments_from_widgets', 10, 2);

/*
 * comment_flood_filter
 */

/**
 * Whether a client's reply should be blocked because of comment flood.
 *
 * @param bool $block Whether a plugin has already blocked the comment.
 * @param int  $time_lastcomment Timestamp for last comment.
 * @param int  $time_newcomment Timestamp for new comment.
 * @return bool Whether comment should be blocked.
 */
function fast_throttle_comment_flood($block, $time_lastcomment, $time_newcomment)
{
    if ($block) {
        return $block;
    }

    if (($time_newcomment - $time_lastcomment) < (int) fast_get_option('ticket_minimum_reply_interval')) {
        return true;
    }
    return false;
}
add_filter('comment_flood_filter', 'fast_throttle_comment_flood', 100, 3);


/*
 * comment_text
 */

/**
 * Makes sure that all ticket comments open in a new tab.
 *
 * @param string $content Comment content.
 * @param object $comment Comment object.
 *
 * @return string Modified comment content.
 */
function comment_links_in_new_tab($content, $comment)
{
    if ('fast_ticket' !== get_post_type($comment->comment_post_ID)) {
        return $content;
    } else {
        return str_replace('<a', '<a target="_blank"', $content);
    }
}
add_filter('comment_text', 'comment_links_in_new_tab', 10, 2);

/*
 * comment_text
 */

/**
 * Adds extra timespans for cron jobs.
 *
 * @param array $schedules Array of schedule times.
 */
function fast_add_extra_cron_schedules($schedules)
{
    $schedules['five_minutes'] = array(
        'interval' => 300,
        'display'  => esc_html__('Every 5 Minutes', 'fast-plugin'),
    );
    $schedules['ten_minutes'] = array(
        'interval' => 600,
        'display'  => esc_html__('Every 10 Minutes', 'fast-plugin'),
    );
    $schedules['fifteen_minutes'] = array(
        'interval' => 900,
        'display'  => esc_html__('Every 15 Minutes', 'fast-plugin'),
    );
    $schedules['half_hour'] = array(
        'interval' => 1800,
        'display'  => esc_html__('Every 30 Minutes', 'fast-plugin'),
    );
    return $schedules;
}
add_filter('cron_schedules', 'fast_add_extra_cron_schedules');
