<?php
/**
 * Actions file for WooCommerce
 *
 * @package Fast_Plugin
 *
 * @copyright 2017 Oxygenna.com
 * @license http://themeforest.net/licenses/standard
 * @version 1.5.0
 * @author Oxygenna
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

 /*
 * init
 */

 /**
  * Adds myaccount link to support tickets if option is on.
  */
function fast_add_support_tickets_to_my_account()
{
    // Only add the support link to myaccounts if option set.
    if ('on' === fast_get_option('woocommerce_support_tickets_link')) {
        add_filter('woocommerce_account_menu_items', 'fast_wc_add_support_link_to_my_account_menu', 10, 1);
        add_filter('woocommerce_get_endpoint_url', 'fast_wc_change_support_ticket_url', 10, 4);
    }

    // Only add get support button if option is set.
    if ('on' === fast_get_option('woocommerce_support_tickets_order_button')) {
        add_filter('woocommerce_my_account_my_orders_actions', 'fast_wc_add_support_link_order_table_actions', 10, 2);
    }
}
add_action('init', 'fast_add_support_tickets_to_my_account');

 /*
 * save_post_fast_ticket
 */

/**
 * Assigns default status to a new ticket
 *
 * @param int  $post_id Post id being saved.
 * @param post $post The post object.
 * @param bool $update Whether this is an existing post being updated or not.
 */
function fast_assign_user_using_woocommerce_on_new_ticket($post_id, $post, $update)
{
    if ((defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) || wp_is_post_revision($post_id)) {
        return;
    }

    if (true === $update) {
        return;
    }

    $agents = fast_get_option('woocommerce_agents');
    if (!empty($agents)) {
        $count_tickets = wp_count_posts('fast_ticket');
        $count = $count_tickets->publish + $count_tickets->private;
        $agent_id = $agents[$count % count($agents)];
        wp_set_object_terms($post_id, intval($agent_id), 'fast_agent', false);
    }
}
add_action('save_post_fast_ticket', 'fast_assign_user_using_woocommerce_on_new_ticket', 10, 3);

 /*
 * woocommerce_created_customer
 */

/**
 * Adds a ticket role when a woocommerce customer is created.
 *
 * @param int    $customer_id New Customer id.
 * @param array  $new_customer_data New Customer Data.
 * @param string $password_generated New customer generated password.
 */
function fast_add_customer_role_on_customer_created($customer_id, $new_customer_data, $password_generated)
{
    $user = get_user_by('id', $customer_id);
    $user->add_role('fast_customer');
}
add_action('woocommerce_created_customer', 'fast_add_customer_role_on_customer_created', 10, 3);
